# File: AnalyzeLog_Perl.pl
# Date            Author        number of nodes
# 2002/10/19      Walking Lint  138
# 2002/7/4        Walking Lint  135
# 2002/4/4        Walking Lint  134
# 2002/3/3        Walking Lint  130
# 2002/3/2        Walking Lint  89
# 2002/2/17       Walking Lint  89
# 2002/2/16       Walking Lint  85
use Time::Local;
$graphlength = 30;
$from_file = shift;
DebugPrint("from_file = $from_file");
open IN, $from_file || die("Can't open $from_file: $!\n");
$logintime = 0;
$newfiletime = 0;
$idledtime = 0;
while (<IN>) {
  if ($_ =~ /idled over ([0-9]+) minutes/) {
    $idledtime = $idledtime  + $1 * 60;
    $idledtime{$year . '/' . $month . '/' . $mday} = $idledtime{$year . '/' . $month . '/' . $mday}  + $1 * 60;
  } elsif ($_ =~ /logged in/) {
    $idledtime = 0;
    $file = $1;
    $_ =~ /([0-9]+)\/([0-9]+)\/([0-9]+) ([0-9]+):([0-9]+):([0-9]+)/;
    $year = $1;
    $month = $2;
    $mday = $3;
    $hours = $4;
    $min = $5;
    $sec = $6;
    $logintime = timelocal($sec, $min, $hours, $mday, $mon, $year);
    $logintime{$year . '/' . $month . '/' . $mday} = timelocal($sec, $min, $hours, $mday, $mon, $year);
  } elsif ($_ =~ /logged out/) {
    $elapsedtimeofday{$year . '/' . $month . '/' . $mday} = $elapsedtimeofday{$year . '/' . $month . '/' . $mday} + timelocal($sec, $min, $hours, $mday, $mon, $year) - $logintime{$year . '/' . $month . '/' . $mday};
    $elapsedtimeofday{$year . '/' . $month . '/' . $mday} -= $idledtime{$year . '/' . $month . '/' . $mday};
    $idledtime{$year . '/' . $month . '/' . $mday} = 0;
  } elsif ($_ =~ /opened file \(([^\)]+)\)/) {
    $idledtime = 0;
    $file = $1;
    $_ =~ /([0-9]+)\/([0-9]+)\/([0-9]+) ([0-9]+):([0-9]+):([0-9]+)/;
    $year = $1;
    $month = $2;
    $mday = $3;
    $hours = $4;
    $min = $5;
    $sec = $6;
    $openedtime{$file} = timelocal($sec, $min, $hours, $mday, $mon, $year);
  } elsif ($_ =~ /new file/) {
    $idledtime = 0;
    $file = $1;
    $_ =~ /([0-9]+)\/([0-9]+)\/([0-9]+) ([0-9]+):([0-9]+):([0-9]+)/;
    $year = $1;
    $month = $2;
    $mday = $3;
    $hours = $4;
    $min = $5;
    $sec = $6;
    $newfiletime = timelocal($sec, $min, $hours, $mday, $mon, $year);
  } elsif ($_ =~ /saved file \(([^\)]+)\)/) {
    $file = $1;
    $_ =~ /([0-9]+)\/([0-9]+)\/([0-9]+) ([0-9]+):([0-9]+):([0-9]+)/;
    $year = $1;
    $month = $2;
    $mday = $3;
    $hours = $4;
    $min = $5;
    $sec = $6;
    $savedtime{$file} = timelocal($sec, $min, $hours, $mday, $mon, $year);
    if (defined($openedtime{$file}) and ($openedtime{$file} > $logintime) and ($openedtime{$file} > $newfiletime) and (($savedtime{$file} - $openedtime{$file} - $idledtime) > 0)) {
      $elapsedtime{$file} = $elapsedtime{$file} + $savedtime{$file} - $openedtime{$file} - $idledtime;
    } elsif (($newfiletime > $logintime) and (($savedtime{$file} - $newfiletime - $idledtime) > 0)) {
      $elapsedtime{$file} = $elapsedtime{$file} + $savedtime{$file} - $newfiletime - $idledtime;
    } elsif (($logintime > 0) and (($savedtime{$file} - $logintime - $idledtime) > 0)) {
      $elapsedtime{$file} = $elapsedtime{$file} + $savedtime{$file} - $logintime - $idledtime;
    }
    $openedtime{$file} = $savedtime{$file}; # this is needed ...
    $idledtime = 0;
  }
}
close IN;
$max = 0;
foreach $key (keys %elapsedtime) {
  $tmp = int($elapsedtime{$key});
  if ($tmp > $max) {
    $max = $tmp;
  }
}
$elemtime = $max / $graphlength;
foreach $key (sort {$elapsedtime{$b} <=> $elapsedtime{$a}} keys %elapsedtime) {
  $hour = int($elapsedtime{$key} / (60 * 60));
  $min = int($elapsedtime{$key} / 60) - $hour * 60;
  $sec = $elapsedtime{$key} - $min * 60 - $hour * 60 * 60;
  $hstr = '';
  $mstr = '';
  $sstr = '';
  if ($hour != 0) {
    $hstr = $hour . 'h';
  }
  if ($min != 0) {
    $mstr = $min . 'm';
  }
  if ($sec != 0) {
    $sstr = $sec . 's';
  }
  $graphline = '*' x (int($elapsedtime{$key} / $elemtime));
  format STDOUT =
@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> - @>>@>>@>> @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$key $hstr $mstr $sstr $graphline
.
  write;
}
print "\n";
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
