# File: CVTree_Perl.pl
# Date            Author        number of nodes
# 2005/3/7        Walking Lint  380
# 2005/3/6        Walking Lint  401
# 2004/3/31       Walking Lint  389
# 2004/1/21       Walking Lint  368
# 2003/10/3       Walking@Lint 370
# 2003/8/19       Walking@Lint 361
# 2003/1/27       Walking Lint  359
# 2002/12/5       Walking Lint  319
# 2002/11/27      Walking Lint  312
# 2002/11/25      Walking Lint  271
# 2002/11/24      Walking Lint  272
# 2002/11/13      Walking Lint  240
# 2002/11/10      Walking Lint  245
# 2002/11/8       Walking Lint  244
# 2002/11/6       Walking Lint  225
# 2002/11/1       Walking Lint  226
# 2002/10/23      Walking Lint  218 (CVTree->CVTree_Perl)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Perl->CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Perl->ExtractCategorizedText_Perl)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Perl->GenCategorizedText_Perl)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

CVTree.pl - package for CV tree

=head1 DESCRIPTION

ex. use lib '.';
    use CVTree;
    my $root = {};
    &initnode($root, 'root');
    &gentree($root, './CVTree.pat');
    &printtree($root, '  ');

=head1 AUTHOR

Walking Lint

=cut

package CVTree_Perl;
use Exporter;
@CVTree_Perl::ISA = ('Exporter');
@CVTree_Perl::EXPORT = ('addchildnode', 'addchildnode2', 'addnode', 'addnode2', 'delnode', 'gentree', 'getnext', 'initnode', 'insnode', 'printtree', 'printtree2', 'savetree', 'sortedsavetree', 'sortedsavetree2');
sub addchildnode {
  my($pn, $name, $kind, $type, $text) = @_;
  if (defined($pn->{CHILD})) {
    $cn = $pn->{CHILD};
    while (defined($cn->{NEXT})) {
      $cn = $cn->{NEXT};
    }
    return addnode($cn, $name, $kind, $type, $text);
  } else {
    $pn->{CHILD} = {};
    &initnode($pn->{CHILD}, $name, $kind, $type, $text);
    $pn->{CHILD}->{PARENT} = $pn;
    return $pn->{CHILD};
  }
}
sub addchildnode2 {
  my($pn, $nd) = @_;
  if (defined($pn->{CHILD})) {
    $cn = $pn->{CHILD};
    while (defined($cn->{NEXT})) {
      $cn = $cn->{NEXT};
    }
    return addnode2($cn, $nd);
  } else {
    $pn->{CHILD} = $nd;
    $pn->{CHILD}->{PARENT} = $pn;
    return $pn->{CHILD};
  }
}
sub addnode {
  my($nd, $name, $kind, $type, $text) = @_;
  my $prevnextnode = $nd->{NEXT};
  $nd->{NEXT} = {};
  &initnode($nd->{NEXT}, $name, $kind, $type, $text);
  $nd->{NEXT}->{PREV} = $nd;
  $nd->{NEXT}->{PARENT} = $nd->{PARENT};
  $nd->{NEXT}->{NEXT} = $prevnextnode;
  if (defined($prevnextnode)) {
    $prevnextnode->{PREV} = $nd->{NEXT};
  }
  return $nd->{NEXT};
}
sub addnode2 {
  my($nd, $nd2) = @_;
  my $prevnextnode = $nd->{NEXT};
  $nd->{NEXT} = $nd2;
  $nd->{NEXT}->{PREV} = $nd;
  $nd->{NEXT}->{PARENT} = $nd->{PARENT};
  $nd->{NEXT}->{NEXT} = $prevnextnode;
  if (defined($prevnextnode)) {
    $prevnextnode->{PREV} = $nd->{NEXT};
  }
  return $nd->{NEXT};
}
sub delnode {
  my($nd) = @_;
  if (not defined($nd->{PREV})) {
    $nd->{PARENT}->{CHILD} = $nd->{NEXT};
  } else {
    $nd->{PREV}->{NEXT} = $nd->{NEXT};
  }
  if (defined($nd->{NEXT})) {
    $nd->{NEXT}->{PREV} = $nd->{PREV};
  }
  $nd->{PARENT} = undef;
  $nd->{PREV} = undef;
  $nd->{NEXT} = undef;
}
sub extractparams {
  my($line) = @_;
  $found_text = 0;
  ($pre, $post) = split(/\(\) \+|\(\) \-|\(\)|\(in\)|\(out\)|\(inout\)/, $line, 2);
  $pre =~ /^([\t]*)(.+)/;
  $level = length($1);
  $name = $2;
  ($kind, $post) = split(/\/\/ /, $post, 2);
  $line =~ /( \(\) \+| \(\) \-| \(\)| \(in\)| \(out\)| \(inout\))/;
  $kind = $1 . ' ' . $kind;
  $kind =~ s/  / /; # this is hack ...
  if ($post =~ /\%\% /) {
    ($type, $text) = split(/\%\% /, $post, 2);
  } else { # not actually ...
    $type = $post;
    $text = '';
  }
  $type =~ s/[ ]+$//;
  $name =~ s/[ ]+$//;
}
sub gentree {
  my($nd, $from_file) = @_;
  # gen tree from fn ...
  $prevlevel = 0;
  my $cur = $nd;
  $first = 1;
  if (length($from_file) != 0) {
    open IN, $from_file || die("Can't open $from_file: $!\n");
    $infilehandle = 'IN'; # see [[SAGE - Perl Practicum - Know All the Angles!http://www.usenix.org/publications/perl/perl02.html]]
  } else {
    $infilehandle = 'STDIN';
  }
  while (<$infilehandle>) {
    chop $_;
    if (/^\t/) {
      &extractparams($_);
      # construct tree here ...
      if ($prevlevel < $level) {
        $cur = addchildnode($cur, $name, $kind, $type, $text);
      } elsif ($prevlevel == $level) {
        $cur = addnode($cur, $name, $kind, $type, $text);
      } else {
        until ($prevlevel == $level) {
          $cur = $cur->{PARENT};
          $prevlevel--;
        }
        $cur = addnode($cur, $name, $kind, $type, $text);
      }
      $prevlevel = $level;
    } elsif ($first) {
      $first = 0;
      &extractparams($_);
      $nd->{NAME} = $name;
      $nd->{TEXT} = $text;
    } else {
      # do nothing ...
    }
  }
  if (length($from_file) != 0) {
    close IN;
  }
}
sub getnext {
  my($nd) = @_;
  if (defined($nd->{CHILD})) {
    return $nd->{CHILD};
  } elsif (defined($nd->{NEXT})) {
    return $nd->{NEXT};
  } else {
    while (defined($nd->{PARENT})) {
      $nd = $nd->{PARENT};
      if (defined($nd->{NEXT})) {
        return $nd->{NEXT};
      }
    }
    return 0;
  }
}
sub initnode {
  my($nd, $name, $kind, $type, $text) = @_;
  $nd->{NAME} = $name;
  $nd->{KIND} = $kind;
  $nd->{TYPE} = $type;
  $nd->{TEXT} = $text;
  $nd->{PARENT} = undef;
  $nd->{CHILD} = undef;
  $nd->{PREV} = undef;
  $nd->{NEXT} = undef;
}
sub insnode {
  my($nd, $name, $kind, $type, $text) = @_;
  my $prevprevnode = $nd->{PREV};
  $nd->{PREV} = {};
  &initnode($nd->{PREV}, $name, $kind, $type, $text);
  $nd->{PREV}->{NEXT} = $nd;
  $nd->{PREV}->{PARENT} = $nd->{PARENT};
  $nd->{PREV}->{PREV} = $prevprevnode;
  if (defined($prevprevnode)) {
    $prevprevnode->{NEXT} = $nd->{PREV};
  } else {
    $nd->{PARENT}->{CHILD} = $nd->{PREV};
  }
  return $nd->{PREV};
}
sub printnode {
  my($nd, $ind) = @_;
  printf STDERR "%sNAME: %s KIND: %s ", $ind, $nd->{NAME}, $nd->{KIND};
  printf STDERR "TYPE: %s\n", $nd->{TYPE};
}
sub printnode2 {
  my($nd, $ind) = @_;
  printf STDERR "%sNAME: %s KIND: %s ", $ind, $nd->{NAME}, $nd->{KIND};
  printf STDERR "TYPE: %s TEXT: %s\n", $nd->{TYPE}, $nd->{TEXT};
}
sub printtree {
  my($nd, $ind, $sep) = @_;
  if (length($sep) == 0) { # this is hack ...
    $ind = '';
    $sep = '  ';
  }
  &printnode($nd, $ind);
  if (defined($nd->{CHILD})) {
    &printtree($nd->{CHILD}, $ind . $sep, $sep);
  }
  if (defined($nd->{NEXT})) {
    &printtree($nd->{NEXT}, $ind, $sep);
  }
}
sub printtree2 {
  my($nd, $ind, $sep) = @_;
  if (length($sep) == 0) { # this is hack ...
    $ind = '';
    $sep = '  ';
  }
  &printnode2($nd, $ind);
  if (defined($nd->{CHILD})) {
    &printtree2($nd->{CHILD}, $ind . $sep, $sep);
  }
  if (defined($nd->{NEXT})) {
    &printtree2($nd->{NEXT}, $ind, $sep);
  }
}
sub savenode {
  my($nd, $ind) = @_;
  if (length($nd->{KIND}) == 0) { # this is hack ... change CVTree for root ...
    $kind = ' ()';
  } else {
    $kind = $nd->{KIND};
  }
  $kind =~ s/[ ]+$//;
  $type = '';
  if ((length($nd->{TYPE}) != 0) or (length($nd->{TEXT}) != 0)) {
    $type = " \/\/ " . $nd->{TYPE};
  }
  $text = '';
  if (length($nd->{TEXT}) != 0) {
    $text = " \%\% " . $nd->{TEXT};
  }
  print $outfilehandle $ind . $nd->{NAME} . $kind . $type . $text . "\n";
}
sub savetree {
  my($nd, $to_file) = @_;
  $sep = "\t";
  if (length($to_file) != 0) {
    open OUT, ">$to_file";
    $outfilehandle = 'OUT';
  } else {
    $outfilehandle = 'STDOUT';
  }
  &savenode($nd, '');
  if (defined($nd->{CHILD})) {
    &savetreebody($nd->{CHILD}, $sep, $sep);
  }
  if (length($to_file) != 0) {
    close OUT;
  }
}
sub savetreebody {
  my($nd, $ind, $sep) = @_;
  &savenode($nd, $ind);
  if (defined($nd->{CHILD})) {
    &savetreebody($nd->{CHILD}, $ind . $sep, $sep);
  }
  if (defined($nd->{NEXT})) {
    &savetreebody($nd->{NEXT}, $ind, $sep);
  }
}
sub sortedsavetree {
  my($nd, $to_file) = @_;
  $sep = "\t";
  if (length($to_file) != 0) {
    open OUT, ">$to_file";
    $outfilehandle = 'OUT';
  } else {
    $outfilehandle = 'STDOUT';
  }
  &savenode($nd, '');
  if (defined($nd->{CHILD})) {
    &sortedsavetreebody($nd->{CHILD}, $sep, $sep);
  }
  if (length($to_file) != 0) {
    close OUT;
  }
}
sub sortedsavetree2 {
  my($nd, $to_file) = @_;
  $sep = "\t";
  if (length($to_file) != 0) {
    open OUT, ">$to_file";
    $outfilehandle = 'OUT';
  } else {
    $outfilehandle = 'STDOUT';
  }
  &savenode($nd, '');
  if (defined($nd->{CHILD})) {
    &sortedsavetreebody2($nd->{CHILD}, $sep, $sep);
  }
  if (length($to_file) != 0) {
    close OUT;
  }
}
sub sortedsavetreebody {
  my($nd, $ind, $sep) = @_;
  my %nodes = ();
  $nodes{$nd->{NAME}} = $nd;
  while (defined($nd->{NEXT})) {
    $nd = $nd->{NEXT};
    $nodes{$nd->{NAME}} = $nd;
  }
  foreach $key (sort {lc($a) cmp lc($b)} keys %nodes) {
    &savenode($nodes{$key}, $ind);
    if (defined($nodes{$key}->{CHILD})) {
      &sortedsavetreebody($nodes{$key}->{CHILD}, $ind . $sep, $sep);
    }
  }
}
sub sortedsavetreebody2 {
  my($nd, $ind, $sep) = @_;
  my %nodes = ();
  $nodes{$nd->{NAME}} = $nd;
  while (defined($nd->{NEXT})) {
    $nd = $nd->{NEXT};
    $nodes{$nd->{NAME} . $nd->{TEXT}} = $nd; # better ?
  }
  foreach $key (sort {lc($a) cmp lc($b)} keys %nodes) {
    &savenode($nodes{$key}, $ind);
    if (defined($nodes{$key}->{CHILD})) {
      &sortedsavetreebody2($nodes{$key}->{CHILD}, $ind . $sep, $sep);
    }
  }
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
1;
