# File: CVTree_Python.py
# Date            Author        number of nodes
# 2003/11/24      Walking@Lint 271
# 2002/11/16      Walking Lint  249
# 2002/11/13      Walking Lint  230
# 2002/11/10      Walking Lint  221
# 2002/11/6       Walking Lint  216
# 2002/11/5       Walking Lint  218
# 2002/11/1       Walking Lint  212
# 2002/10/30      Walking Lint  206
# 2002/10/30      Walking Lint  218 (CVTree_Ruby.CVTree_Python)
# 2002/10/23      Walking Lint  218 (CVTree.CVTree_Ruby)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Ruby.CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Ruby.ExtractCategorizedText_Ruby)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Ruby.GenCategorizedText_Ruby)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85
# NAME         CVTree_Python.py - package for CV tree
# DESCRIPTION  ex. root = {}
#                  initnode(root, 'root', '', '', '')
#                  gentree(root, './Memo.pat')
#                  printtree(root, '', '  ')
# AUTHOR       [[Author]]
import re
import string
import sys
def addchildnode(pn, name, kind, type, text):
    if pn['CHILD'] != None:
      cn = pn['CHILD']
      while cn['NEXT'] != None:
        cn = cn['NEXT']
      return addnode(cn, name, kind, type, text)
    else:
      pn['CHILD'] = {}
      initnode(pn['CHILD'], name, kind, type, text)
      pn['CHILD']['PARENT'] = pn
      return pn['CHILD']
def addnode(nd, name, kind, type, text):
    prevnextnode = nd['NEXT']
    nd['NEXT'] = {}
    initnode(nd['NEXT'], name, kind, type, text)
    nd['NEXT']['PREV'] = nd
    nd['NEXT']['PARENT'] = nd['PARENT']
    nd['NEXT']['NEXT'] = prevnextnode
    if prevnextnode != None:
      prevnextnode['PREV'] = nd
    return nd['NEXT']
def delnode(nd):
    if nd['PREV'] != None:
      nd['PREV']['NEXT'] = nd['NEXT']
      if nd['NEXT'] != None:
        nd['NEXT']['PREV'] = nd['PREV']
    else:
      if nd['PARENT'] != None:
        nd['PARENT']['CHILD'] = nd['NEXT']
    nd['PARENT'] = None
    nd['PREV'] = None
    nd['NEXT'] = None
def extractparams(line):
    found_text = 0
    p = re.compile("\(\) \+|\(\) \-|\(\)|\(in\)|\(out\)|\(inout\)")
    pre, post = p.split(line, 2)
    m = re.match("^([\t]*)(.+)", pre)
    level = len(m.group(1))
    name = m.group(2)
    p = re.compile("\/\/ ")
    if p.search(post):
      kind, post = p.split(post, 2)
    else:
      kind = post
      post = ''
    m = re.search("(\(\) \+|\(\) \-|\(\)|\(in\)|\(out\)|\(inout\))", line)
    kind = m.group(1) + ' ' + kind
    kind = re.sub('  ', ' ', kind)
    p = re.compile("\%\% ")
    if p.search(post):
      p = re.compile("\%\% ")
      type, text = p.split(post, 2)
    else: # not actually ...
      type = post
      text = ''
    type = re.sub("[ ]+$", '', type)
    name = re.sub("[ ]+$", '', name)
    return [level, name, kind, type, text]
def gentree(nd, from_file):
    if len(from_file) != 0:
      infile = open(from_file, "r")
    else:
      infile = sys.stdin
    # gen tree from fn ...
    prevlevel = 0
    cur = nd
    first = 1
    for line in infile:
      line = line[:-1]
      p = re.compile("^\t")
      if p.search(line):
        level, name, kind, type, text = extractparams(line)
        # construct tree here ...
        if prevlevel < level:
          cur = addchildnode(cur, name, kind, type, text)
        elif prevlevel == level:
          cur = addnode(cur, name, kind, type, text)
        else:
          while not prevlevel == level:
            cur = cur['PARENT']
            prevlevel -= 1
          cur = addnode(cur, name, kind, type, text)
        prevlevel = level
      elif first:
        first = 0
        level, name, kind, type, text = extractparams(line)
        nd['TEXT'] = text
    if len(from_file) != 0:
      infile.close
def initnode(nd, name, kind, type, text):
    nd['NAME'] = name
    nd['KIND'] = kind
    nd['TYPE'] = type
    nd['TEXT'] = text
    nd['PARENT'] = None
    nd['CHILD'] = None
    nd['PREV'] = None
    nd['NEXT'] = None
def printnode(nd, ind):
    sys.stderr.write("%sNAME: %s KIND: %s " % (ind, nd['NAME'], nd['KIND']))
    sys.stderr.write("TYPE: %s TEXT: %s\n" % (nd['TYPE'], nd['TEXT']))
def insnode(nd, name, kind, type, text):
    prevprevnode = nd['PREV']
    nd['PREV'] = {}
    initnode(nd['PREV'], name, kind, type, text)
    nd['PREV']['NEXT'] = nd
    nd['PREV']['PARENT'] = nd['PARENT']
    nd['PREV']['PREV'] = prevprevnode
    if prevprevnode != None:
      prevprevnode['NEXT'] = nd['PREV']
    else:
      nd['PARENT']['CHILD'] = nd['PREV']
    return nd['PREV']
def printtree(nd, ind, sep):
    printnode(nd, ind)
    if nd['CHILD'] != None:
      printtree(nd['CHILD'], ind + sep, sep)
    if nd['NEXT'] != None:
      printtree(nd['NEXT'],  ind, sep)
def savenode(nd, ind, outfile):
    if len(nd['KIND']) == 0: # this is hack ... change CVTree for root ...
      kind = ' ()'
    else:
      kind = nd['KIND']
    kind = re.sub("[ ]+$", '', kind)
    type = ''
    if (len(nd['TYPE']) != 0) or (len(nd['TEXT']) != 0):
      type = " // " + nd['TYPE']
    text = ''
    if len(nd['TEXT']) != 0:
      text = " %" + "% " + nd['TEXT']
    outfile.write("%s%s%s%s%s\n" % (ind, nd['NAME'], kind, type, text))
def savetree(nd, to_file):
    sep = "\t"
    if len(to_file) != 0:
      outfile = open(to_file, "w")
    else:
      outfile = sys.stdout
    savenode(nd, '', outfile)
    if nd['CHILD'] != None:
      savetreebody(nd['CHILD'], sep, sep, outfile)
    if len(to_file) != 0:
      outfile.close
def savetreebody(nd, ind, sep, outfile):
    savenode(nd, ind, outfile)
    if nd['CHILD'] != None:
      savetreebody(nd['CHILD'], ind + sep, sep, outfile)
    if nd['NEXT'] != None:
      savetreebody(nd['NEXT'],  ind, sep, outfile)
def DebugPrint(str):
    sys.stderr.write(str + "\n")
if __name__ == "__main__":
    root = {}
    initnode(root, 'root', '', '', '')
    gentree(root, './Memo.pat')
    printtree(root, '', '  ')
