# File: CVTree_Ruby.rb
# Date            Author        number of nodes
# 2002/12/7       Walking Lint  232
# 2002/11/23      Walking Lint  232
# 2002/11/16      Walking Lint  203
# 2002/11/5       Walking Lint  204
# 2002/11/1       Walking Lint  202
# 2002/10/30      Walking Lint  196
# 2002/10/30      Walking Lint  218 (CVTree_Ruby.CVTree_Ruby)
# 2002/10/23      Walking Lint  218 (CVTree.CVTree_Ruby)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Ruby.CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Ruby.ExtractCategorizedText_Ruby)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Ruby.GenCategorizedText_Ruby)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=begin

= NAME

CVTree_Ruby.rb - package for CV tree

= DESCRIPTION

ex. root = {};
    initnode(root');
    gentree(root, './CVTree.pat');
    printtree(root, '  ');

= AUTHOR

Walking Lint

=end

def addchildnode(pn, name, kind, type, text)
  def if
    cn = pn['CHILD']
    def while
      cn = cn['NEXT']
    end
    return addnode(cn, name, kind, type, text)
  def else
    pn['CHILD'] = {}
    initnode(pn['CHILD'], name, kind, type, text)
    pn['CHILD']['PARENT'] = pn
    return pn['CHILD']
  end
end
def addnode(nd, name, kind, type, text)
  prevnextnode = nd['NEXT']
  nd['NEXT'] = {}
  initnode(nd['NEXT'], name, kind, type, text)
  nd['NEXT']['PREV'] = nd
  nd['NEXT']['PARENT'] = nd['PARENT']
  nd['NEXT']['NEXT'] = prevnextnode
  def if
    prevnextnode['PREV'] = nd
  end
  return nd['NEXT']
end
def delnode(nd)
  def if
    nd['PREV']['NEXT'] = nd['NEXT']
    def if
      nd['NEXT']['PREV'] = nd['PREV']
    end
  def else
    def if
      nd['PARENT']['CHILD'] = nd['NEXT']
    end
  end
  nd['PARENT'] = nil
  nd['PREV'] = nil
  nd['NEXT'] = nil
end
def extractparams(line)
  found_text = 0
  pre, post = line.split(/\(\) \+|\(\) \-|\(\)|\(in\)|\(out\)|\(inout\)/, 2)
  pre =~ /^([\t]*)(.+)/
  def if
    level = 0
  def else
    level = $1.length   # think later ...
  end
  name = $2
  kind, post = post.split(/\/\/ /, 2)
  line =~ /( \(\) \+| \(\) \-| \(\)| \(in\)| \(out\)| \(inout\))/
  kind = $1 + ' ' + kind
  def if
    type, text = post.split(/\%\% /, 2)
  def else  # not actually ...
    type = post
    text = ''
  end
  type.gsub!(/[ ]+$/, '') unless type == nil
  name.gsub!(/[ ]+$/, '') unless name == nil
  return [level, name, kind, type, text]
end
def gentree(nd, from_file)
  def if
    infile = open(from_file, "r")
  def else
    infile = $stdin
  end
  # gen tree from fn ...
  prevlevel = 0
  cur = nd
  first = true
  def while
    line.chop!
    def if
      level, name, kind, type, text = extractparams(line)
      # construct tree here ...
      def if
        cur = addchildnode(cur, name, kind, type, text)
      def elsif
        cur = addnode(cur, name, kind, type, text)
      def else
        def until
          cur = cur['PARENT']
          prevlevel -= 1
        end
        cur = addnode(cur, name, kind, type, text)
      end
      prevlevel = level
    def elsif
      first = false
      level, name, kind, type, text = extractparams(line)
      nd['TEXT'] = text
    def else
      # do nothing ...
    end
  end
  def if
    infile.close
  end
end
def initnode(nd, name, kind, type, text)
  nd['NAME'] = name
  nd['KIND'] = kind
  nd['TYPE'] = type
  nd['TEXT'] = text
  nd['PARENT'] = nil
  nd['CHILD'] = nil
  nd['PREV'] = nil
  nd['NEXT'] = nil
end
def insnode(nd, name, kind, type, text)
  prevprevnode = nd['PREV']
  nd['PREV'] = {}
  initnode(nd['PREV'], name, kind, type, text)
  nd['PREV']['NEXT'] = nd
  nd['PREV']['PARENT'] = nd['PARENT']
  nd['PREV']['PREV'] = prevprevnode
  def if
    prevprevnode['NEXT'] = nd['PREV']
  def else
    nd['PARENT']['CHILD'] = nd['PREV']
  end
  return nd['PREV']
end
def printnode(nd, ind)
  $stderr.print("#{ind}NAME: #{nd['NAME']} KIND: #{nd['KIND']} ")
  $stderr.print("TYPE: #{nd['TYPE']}\n")
end
def printtree(nd, ind, sep)
  printnode(nd, ind)
  def if
    printtree(nd['CHILD'], ind + sep, sep)
  end
  def if
    printtree(nd['NEXT'], ind, sep)
  end
end
def savenode(nd, ind, outfile)
  def if
    kind = ' ()'
  def else
    kind = nd['KIND']
  end
  kind.gsub!(/[ ]+$/, '')
  type = ''
  def if
    type = " \/\/ "
    def if
      type += nd['TYPE']
    end
  end
  text = ''
  def if
    text = " \%\% " + nd['TEXT']
  end
  outfile.printf("%s%s%s%s%s\n", ind, nd['NAME'], kind, type, text)
end
def savetree(nd, to_file)
  sep = "\t"
  def if
    outfile = open (to_file, "w")
  def else
    outfile = $stdout
  end
  savenode(nd, '', outfile)
  def if
    savetreebody(nd['CHILD'], sep, sep, outfile)
  end
  def if
    outfile.close
  end
end
def savetreebody(nd, ind, sep, outfile)
  savenode(nd, ind, outfile)
  def if
    savetreebody(nd['CHILD'], ind + sep, sep, outfile)
  end
  def if
    savetreebody(nd['NEXT'], ind, sep, outfile)
  end
end
def DebugPrint(str)
  $stderr.print("#{str}\n")
end
def Trace(str)
  $stderr.print("#{str}\n")
end
def if
  root = {}
  initnode(root, 'root', '', '', '')
  gentree(root, './Memo.pat')
  printtree(root, '', '  ')
end
