# File: CopyFilesToUpload_Perl.pl
# Date            Author        number of nodes
# 2003/8/27       Walking@Lint 95
# 2003/8/8        Walking@Lint 95
# 2003/8/4        Walking@Lint 95
# 2003/7/20       Walking Lint  92
# 2003/5/15       Walking Lint  88
# 2003/5/14       Walking Lint  86
# 2003/5/4        Walking Lint  77
# 2003/3/18       Walking Lint  74
# 2003/3/16       Walking Lint  72
# 2002/11/29      Walking Lint  72
# 2002/11/27      Walking Lint  70
# 2002/11/23      Walking Lint  70
# 2002/11/6       Walking Lint  61
# 2002/10/31      Walking Lint  61
# 2002/10/11      Walking Lint  59
# 2002/9/18       Walking Lint  49
# 2002/9/7        Walking Lint  49
# 2002/7/23       Walking Lint  9 (CheckModifiedTime_Perl->CopyFilesToUpload_Perl)
# 2002/6/4        Walking Lint  6 (Time_Perl->CheckModifiedTime_Perl)
# 2002/3/3        Walking Lint  3 (Time_Perl->NewFile)
# 2002/2/19       Walking Lint  3
# 2002/2/17       Walking Lint  3

=pod

=head1 NAME

CopyFilesToUpload_Perl.pl - copy files generated after ftpx's last activation time

=head1 DESCRIPTION

uncheck ftpx's "View > Options > Paths > Clean up temporary directory on shutdown" and execute ftpx once
(the files in the temporary directory is used to determine last activated time)

ex1. perl.exe CopyFilesToUpload_Perl.pl "D:\HP\vector" "D:\HP\vector\upload"

ex2. perl.exe CopyFilesToUpload_Perl.pl "D:\tmp\cv\templates" "D:\tmp\cv\templates\upload"

=head1 AUTHOR

Walking Lint

=cut

use File::Basename;
use Win32::File;
use File::Copy;
use Time::localtime;
use Win32::Registry;
$from_dir = shift;
DebugPrint("from_dir = $from_dir");
$to_dir = shift;
DebugPrint("to_dir = $to_dir");
if (not -e $to_dir) {
  mkdir $to_dir, 0700 or die("Cannot make $to_dir: $!");
  DebugPrint("generated $to_dir");
}
$HKEY_CURRENT_USER->Open("SOFTWARE\\FTP Explorer\\FTP Explorer\\Settings", $ref) or die "cannot open registry: $!";
$ref->GetValues(\%values);
$ref->Close();
$ref_ftpx_tmp_dir = $values{'TmpPath'};
$ftpx_tmp_dir = $$ref_ftpx_tmp_dir[2];
$ftpx_tmp_dir =~ s/\\$//;
if (length($ftpx_tmp_dir) == 0) {
  $ftpx_tmp_dir = $ENV{FTPTMPDIR}; # this is needed for XP ???
  $ftpx_tmp_dir =~ s/"//g;
}
DebugPrint("ftpx_tmp_dir = " . $ftpx_tmp_dir);
# get ftpx's last activation time
  if (not -e $ftpx_tmp_dir) {
    mkdir $ftpx_tmp_dir, 0700 or die("Cannot make $ftpx_tmp_dir: $!");
  }
opendir DIR, $ftpx_tmp_dir or die "opendir $ftpx_tmp_dir failed: $!";
@files = grep /\.tmp$/i, readdir DIR;
closedir DIR;
$last_time = 0;
  foreach $from_file (@files) {
    $from_file = $ftpx_tmp_dir . "\\" . basename($from_file);
    @stat = stat($from_file);
    if ($stat[9] > $last_time) {
      $last_time = $stat[9];
    }
  }
  foreach $from_file (@files) {
    $from_file = $ftpx_tmp_dir . "\\" . basename($from_file);
    @stat = stat($from_file);
    if ($stat[9] != $last_time) {
      unlink $from_file;
    }
  }
DebugPrint("last_time = " . &gendate($last_time));
# clear upload dir
opendir DIR, $to_dir or die "opendir $to_dir failed: $!";
@files = readdir DIR;
closedir DIR;
  foreach $from_file (@files) {
    $from_file = $to_dir . "\\" . basename($from_file);
    unlink $from_file;
  }
# copy files modified after ftpx's last activation
opendir DIR, $from_dir or die "opendir $from_dir failed: $!";
@files = grep /\.(class|css|gif|html|jpg|mpg|pat|svg|txt|wrl|xml|zip)$/i, readdir DIR;
%dates = ();
closedir DIR;
  foreach $from_file (@files) {
    $from_file = $from_dir . "\\" . basename($from_file);
    @stat = stat($from_file);
    if ((($stat[9] > $last_time) or ($stat[10] > $last_time)) and not (basename($from_file) eq 'copyright.html') and not (basename($from_file) eq 'menu.html')) {
      $to_file = $to_dir . "\\" . basename($from_file);
      copy($from_file, $to_file);
      $dates{$to_file} = &gendate($stat[9]);
    }
  }
$first = 1;
  foreach $key (sort {$dates{$a} cmp $dates{$b}} keys %dates) {
    if ($first) {
      $first = 0;
      DebugPrint('');
    }
    DebugPrint("copied to $key - $dates{$key}");
  }
sub gendate {
  my($time) = @_;
  $tm = localtime($time);
  $year = $tm->year + 1900;
  $month = $tm->mon + 1;
  $day = $tm->mday;
  $hour = $tm->hour;
  $min = $tm->min;
  $sec = $tm->sec;
  return sprintf("$year/%02d/%02d $hour:$min:$sec", $month, $day);
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
