# File: DotWriter_Perl.pl
# Date            Author        number of nodes
# 2005/4/28       Walking Lint  95
# 2005/4/13       Walking Lint  88
# 2005/4/7        Walking Lint  78
# 2005/4/4        Walking Lint  71
# 2005/4/2        Walking Lint  71
# 2003/10/20      Walking Lint  61
# 2003/10/20      Walking Lint  44 (IniWriter_Perl->DotWriter_Perl)
# 2003/9/2        Walking Lint  44
# 2003/3/6        Walking Lint  126
# 2003/3/3        Walking Lint  120
# 2003/2/13       Walking Lint  219 (GenRSS_Perl->IniWriter_Perl)
# 2002/11/25      Walking Lint  215
# 2002/11/25      Walking Lint  212 (GenRSS_Perl->NewFile)
# 2002/11/18      Walking Lint  212
# 2002/11/15      Walking Lint  212
# 2002/11/5       Walking Lint  212
# 2002/11/1       Walking Lint  205
# 2002/10/31      Walking Lint  184
# 2002/10/28      Walking Lint  161
# 2002/10/22      Walking Lint  138
# 2002/10/18      Walking Lint  136
# 2002/10/17      Walking Lint  76 (GenRSS_Perl->GenRSS_Perl)
# 2002/10/11      Walking Lint  178 (GenCategorizedLinkLog_Perl->GenRSS_Perl)
# 2002/10/10      Walking Lint  178
# 2002/10/10      Walking Lint  172 (GenCategorizedLinkLogNew_Perl->GenCategorizedLinkLog_Perl)
# 2002/9/21       Walking Lint  109 (GenCategorizedLinkLog_Perl->GenCategorizedLinkLogNew_Perl)
# 2002/9/3        Walking Lint  149
# 2002/7/4        Walking Lint  141
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

DotWriter_Perl.pl - .dot (for Graphviz) writer

=head1 DESCRIPTION

ex1. .dot () // DotReader_Perl %% DotWriter_Perl
ex2. _presave ()
     _script () // DotWriter_Perl C:\memo\CV\templates\DotWriter_Perl.pat 1

see. <a href="http://mojix.org/2003/10/19/1743">ZopeWL[L : Graphviz - open source graph drawing software</a>

see. <a href="http://tmasada.exblog.jp/2489652/">photogenic blue note</a>

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use File::Basename;
use Time::localtime;
use Jcode;
$threshold = 3;
$to_file = shift;
$from_file = $to_file;
$from_file =~ s/\..+/.pat/;
my $root = {};
&initnode($root, 'root', '', '', '');
&gentree($root, $from_file);
open OUT, ">$to_file";
$crlf = ' CR' . 'LF ';
$found_child = 0;
$nd = $root->{CHILD};
while (not ($nd eq undef)) {
  if (defined($nd->{CHILD})) {
    $found_child = 1;
    last;
  }
  $nd = $nd->{NEXT};
}
if ($found_child == 0) {
  print OUT <<"END";
digraph g {
node [fontname = "hgrsmp"]
END
}
%count = ();
&gendot($root, 0);
if ($found_child == 0) {
  # now ... this case only ... think later ...
  foreach $key (sort keys %count) {
    if ($count{$key} > $threshold) { # ex. 3
      Jcode::convert(\$key, 'utf8', 'sjis', "z");
      print OUT $key . '[shape=rectangle];' . "\n";
    }
  }
  print OUT '}' . "\n"; # this is hack ...
}
close OUT;
sub gendot {
  my($nd, $level) = @_;
  my $indent = '  ' x $level;
  if (($nd->{NAME} =~ /^_/) and not ($nd->{NAME} eq '_text')) { # this is needed ... think later ...
    return;
  } elsif ($nd->{NAME} eq 'root') {
    $level = -1;
  }
  if (defined($nd->{CHILD})) {
    if ($level == -1) { # do nothing ... for root ...
    } elsif ($level == 0) {
      print OUT $indent . 'digraph ' . $nd->{NAME} . ' {' . "\n"; # think later ...
    } else {
      print OUT $indent . 'subgraph ' . $nd->{NAME} . ' {' . "\n";
    }
    my $cn = $nd->{CHILD};
    while (1) {
      gendot($cn, $level + 1);
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
    if ($level == -1) { # do nothing ... for root ...
    } else {
      print OUT $indent . '}' . "\n";
    }
  } else {
    if ($level == -1) { # do nothing ... for root ...
    } else {
      if ($nd->{NAME} eq '_text') {
        $text = $nd->{TEXT};
        $text =~ s/$crlf/\n/;
        $text =~ s/[\n]+$//;
        $line = "$text\n";
      } elsif ($nd->{KIND} eq ' () ') {
        if (length($nd->{TYPE}) == 0) {
          $line = $nd->{NAME} . ' -> ' . $nd->{TEXT} . ";\n";
          $count{$nd->{TEXT}}++;
        } else {
          if (length($nd->{TEXT}) != 0) {
            $line = $nd->{NAME} . ' -> ' . $nd->{TYPE} . " $nd->{TEXT};\n";
          } else {
            $line = $nd->{NAME} . ' -> ' . $nd->{TYPE} . ";\n";
          }
          $count{$nd->{TYPE}}++;
        }
      } else {
        $line = $nd->{NAME} . ' = ' . $nd->{TYPE} . ";\n";
      }
      Jcode::convert(\$line, 'utf8', 'sjis', "z");
      print OUT $indent . $line;
    }
  }
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
sub Trace {
  my($str) = @_;
  print STDERR "$str\n";
}
