# File: EARReader_Perl.pl
# Date            Author        number of nodes
# 2003/10/3       Walking@Lint 140
# 2003/7/2        Walking Lint  132
# 2003/7/1        Walking Lint  124
# 2003/7/1        Walking Lint  101 (GeppouReader_Perl->EARReader_Perl)
# 2003/6/30       Walking Lint  101
# 2002/12/3       Walking Lint  98
# 2002/12/2       Walking Lint  93
# 2002/11/24      Walking Lint  315 (PythonReader_Perl->GeppouReader_Perl)
# 2002/11/24      Walking Lint  287 (RubyReader_Perl->PythonReader_Perl)
# 2002/11/23      Walking Lint  173
# 2002/11/13      Walking Lint  240 (CVTree_Perl->RubyReader_Perl)
# 2002/11/10      Walking Lint  245
# 2002/11/8       Walking Lint  244
# 2002/11/6       Walking Lint  225
# 2002/11/1       Walking Lint  226
# 2002/10/23      Walking Lint  218 (CVTree->CVTree_Perl)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Perl->CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Perl->ExtractCategorizedText_Perl)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Perl->GenCategorizedText_Perl)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

EARReader_Perl.pl - EAR reader see. http://w3.access.gpo.gov/bis/ear/ear_data.html

=head1 DESCRIPTION

activate as filter

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
$from_file = shift;
DebugPrint("from_file = $from_file");
$to_file = $from_file;
$to_file =~ s/\..+/.pat/;
my $root = {};
$first = 1;
&initnode($root, 'root');
&gentree($root, $from_file);
&savetree($root, $to_file);
DebugPrint("generated $to_file");
sub extractparams {
  my($line) = @_;
  $kind = ' ()  '; # somehow this is hack ... think later ...
  $type = '';
  $text = '';
  if ($first and ($line =~ /^[ \t]*(Part 7[0-9]{2})/i)) {
    $level = 0;
    $name = $1;
    $founditem = 1;
    $first = 0;
    return 0;
  } elsif ($line =~ /^[ \t]*[\xa7\x15]+(7[0-9]{2}\.[0-9]+)[ \t\l\r]*$/) {
    $level = 1;
    $name = $1;
    $founditem = 1;
    return 0;
  } elsif ($line =~ /^[ \t]*(\([0-9]+\).*)/) {
    $level = 3;
    $name = $1;
    $founditem = 1;
    return 0;
  } elsif ($line =~ /^[ \t]*(\([ivx]+\).*)/) {
    $level = 4;
    $name = $1;
    $founditem = 1;
    return 0;
  } elsif ($line =~ /^[ \t]*(\([a-z]\).*)/) {
    $level = 2;
    $name = $1;
    $founditem = 1;
    return 0;
  } elsif ($line =~ /^[ \t]*(\([A-Z]\).*)/) {
    $level = 5;
    $name = $1;
    $founditem = 1;
    return 0;
  } else {
    return 1;
  }
}
sub gentree {
  my($nd, $from_file) = @_;
  # gen tree from fn ...
  $prevlevel = -1;
  @prevlevels = ();
  push @prevlevels, $prevlevel; # not sure if needed ...
  my $cur = $nd;
  $founditem = 0;
  if (length($from_file) != 0) {
    open IN, $from_file || die("Can't open $from_file: $!\n");
    $infilehandle = 'IN'; # see [[SAGE - Perl Practicum - Know All the Angles!http://www.usenix.org/publications/perl/perl02.html]]
  } else {
    $infilehandle = 'STDIN';
  }
  $breaklater = 0;
  while (<$infilehandle>) {
    if ($_ =~ /^__EOF__[\n\r]*$/) {
      last;
    } elsif ($_ =~ /__EOF__[\n\r]*$/) {
      $_ =~ s/__EOF__[\n\r]*$//;
      $breaklater = 1;
    }
    $_ =~ s/[\n\r\l]+$//;
    if ((length($_) != 0)) {
      if (&extractparams($_)) {
        if ($founditem) {
          $_ =~ s/^[ @\t\l\r]*//;
          if (length($cur->{TEXT}) == 0) {
            if (length($_) != 0) {
              $cur->{TEXT} = $_;
            }
          } else {
            $tmpline = " \CRLF " . $_;
            $cur->{TEXT} .= $tmpline;
          }
        }
        next;
      }
      # construct tree here ...
      if ((length($name) > 250) and ($name =~ /^([^ \t]+)[ \t]+(.*)$/)) {
        $text = $2;
        $name = $1;
      }
      $name =~ s/[ \t]+/ /g;
      $text =~ s/^[ \t]+//;
      if ($prevlevel < $level) {
        push @prevlevels, $prevlevel;
        for ($i = 1; $i < $level - $prevlevel; $i++) {
          $cur = addchildnode($cur, '', ' ()  ', '', '');
        }
        $cur = addchildnode($cur, $name, $kind, $type, $text);
      } elsif ($prevlevel == $level) {
        $cur = addnode($cur, $name, $kind, $type, $text);
      } else {
        until ($prevlevel <= $level) {
          $cur = $cur->{PARENT};
          $prevlevel = pop @prevlevels;
        }
        $cur = addnode($cur, $name, $kind, $type, $text);
      }
      $prevlevel = $level;
    }
    if ($breaklater) {
      last;
    }
  }
  if (length($from_file) != 0) {
    close IN;
  }
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
