# File: ExcludeCommentedOutImageFiles_Perl.pl
# Date            Author        number of nodes
# 2006/6/27       Walking Lint  67
# 2006/6/26       Walking Lint  67
# 2006/6/24       Walking Lint  66

=pod

=head1 NAME

ExcludeCommentedOutImageFiles_Perl.pl

=head1 DESCRIPTION

can be used to clean-up commented out image files in current folder

=head1 AUTHOR

Walking Lint

=cut

use File::Copy;
$from_file = shift;
$backup_folder = shift;
$check_only = shift;
$commented_out = 0;
$indent = '';
%commentedout_files = ();
%not_commentedout_files = ();
@exclude_files = ();
open IN, $from_file or die("Cannot open $from_file: $!\n");
while (<IN>) {
  if ($commented_out == 0) {
    if ($_ =~ /^([\t]*)_note \(\) \/\/ commented out/) {
      $indent = $1 . "\t";
      $commented_out = 1;
    }
  } else {
    if ($_ =~ /^$indent/) {
      # commented out
    } elsif ($_ =~ /^([\t]*)_note \(\) \/\/ commented out/) {
      $indent = $1 . "\t";
    } else {
      $commented_out = 0;
    }
  }
  if ($commented_out) {
    if ($_ =~ /_part \(\) (?:[^\/]*)\/\/ (.*\.jpg)/) {
      $commentedout_files{$1}++;
    }
  } else {
    if ($_ =~ /_part \(\) (?:[^\/]*)\/\/ (.*\.jpg)/) {
      $not_commentedout_files{$1}++;
    }
    if ($_ =~ /_text \(\) (?:[^\/]*)\/\/(?: )+\%\% (.*)/) {
      $text = $1;
      while ($text =~ /(?:src|href)="([^"]*)"/g) {
        $matched = $1;
        $not_commentedout_files{$matched}++;
      }
    }
  }
}
close IN;
foreach $commentedout_file (keys %commentedout_files) {
  if (defined($not_commentedout_files{$commentedout_file})) {
    # there was not commented out node
  } else {
    $image_file = $from_file;
    $image_file =~ s/\\[^\\]*$/\\$commentedout_file/;
    if (-e $image_file) {
      DebugPrint("found $image_file to exclude");
      push @exclude_files, $commentedout_file;
    }
  }
}
if ($check_only != 1) {
  if (not -e $backup_folder) {
    mkdir $backup_folder, 0700 or die("Cannot make $backup_folder: $!");
  }
  foreach $exclude_file (@exclude_files) {
    $to_file = $backup_folder . '\\' . $exclude_file;
    $image_file = $from_file;
    $image_file =~ s/\\[^\\]*$/\\$exclude_file/;
    move($image_file, $to_file);
    DebugPrint("moved $image_file to $to_file");
  }
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
