# File: GenColorTable_Perl.pl
# Date            Author        number of nodes
# 2003/5/6        Walking Lint  145
# 4003/5/4        Walking Lint  148
# 4003/4/11       Walking Lint  143
# 4003/4/10       Walking Lint  124
# 4003/3/23       Walking Lint  119
# 4002/11/21      Walking Lint  116
# 4002/11/16      Walking Lint  115
# 4002/10/19      Walking Lint  115
# 4002/10/15      Walking Lint  81 (GenColorTable_Perl->GenColorTable_Perl)
# 4002/9/21       Walking Lint  168
# 4002/8/31       Walking Lint  156

=pod

=head1 NAME

GenColorTable_Perl.pl - generate ColorTable section and ColorTable.html

=head1 DESCRIPTION

ex. perl.exe GenColorTable

meaning of flags in kind

! - exclude

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use Time::localtime;
# set parameters
$templatesfolder = $ENV{TEMPLATESFOLDER};
$templatesfolder =~ s/"//g;
$colors_file = $templatesfolder . '\Colors.pat';
DebugPrint("colors_file = $colors_file");
$colortable_file = $templatesfolder . '\ColorTable.html';
DebugPrint("colortable_file = $colortable_file");
$CSSTemplate = $templatesfolder . '\ColorTable.css';
DebugPrint("CSSTemplate = $CSSTemplate");
$weblogfolder = $ENV{WEBLOGFOLDER};
$weblogfolder =~ s/"//g;
$copyright_file = $weblogfolder . '\copyright.html';
open IN, $copyright_file || die("Cannot open $copyright_file: $!\n");
@copyrights = <IN>;
close IN;
$copyright = join('', @copyrights);
$copyright =~ s/\n$//;
DebugPrint("copyright = $copyright");
my $root = {};
&initnode($root, 'root', '', '', '');
&gentree($root, $colors_file);
&gencolortable($root, $colortable_file);
DebugPrint("generated $colortable_file");
sub gencolortable {
  my($nd, $to_file) = @_;
  if (defined($nd->{CHILD})) {
    my $first = 1;
    $cn = $nd->{CHILD};
    while (1) {
      if ($cn->{NAME} =~ /^_/) {
        # skip
      } else {
        if ($first) {
          $first = 0;
          open OUT, ">$to_file";
          $date = &gendate(time);
          if (length($CSSTemplate) != 0 <<END) {
            print OUT <<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN">
<html>
<head>
<title>color table</title>
<meta http-equiv="content-style-type" content="text/css" />
<link rel="stylesheet" type="text/css" href="$CSSTemplate" />
</head>
<body>
<h1>Color Table</h1>
<div class="lastupdate">Last update: $date</div>
<center>
<table>
END
          } else {
            print OUT <<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN">
<html>
<head>
<title>color table</title>
</head>
<body>
<h1>Color Table</h1>
<div class="lastupdate">Last update: $date</div>
<center>
<table>
END
          }
        }
        $Sub = '';
        $SubFont = '';
        $Accent = '';
        $AccentFont = '';
        $Main = '';
        $MainFont = '';
        $Link = '';
        $Visited = '';
        $Background = '';
        if (defined($cn->{CHILD})) {
          $ccn = $cn->{CHILD};
          while (1) {
            DebugPrint("ccn = $ccn->{NAME}");
            if ($ccn->{NAME} eq 'Sub') {
              $Sub = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'SubFont') {
              $SubFont = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'Accent') {
              $Accent = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'AccentFont') {
              $AccentFont = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'Main') {
              $Main = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'MainFont') {
              $MainFont = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'Link') {
              $Link = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'Visited') {
              $Visited = $ccn->{TYPE};
            } elsif ($ccn->{NAME} eq 'Background') {
              $Background = $ccn->{TYPE};
            }
            if (not defined($ccn->{NEXT})) {
              last;
            }
            $ccn = $ccn->{NEXT};
          }
        }
        if (length($Link) == 0) {
          $Link = '#0000ff';
        }
        if (length($Visited) == 0) {
          $Visited = '#80080';
        }
        if (length($Background) == 0) {
          $Background = black;
        }
        print OUT <<END;
<tr>
<td bgcolor="$Sub" width="400"><font color="$SubFont">$cn->{NAME}(Sub)</font> <font color="$Link"><u>Link</u></font> <font color="$Visited"><u>Visited</u></font></td>
<td bgcolor="$Main" width="400"><font color="$MainFont">$cn->{NAME}(Main)</font> <font color="$Link"><u>Link</u></font> <font color="$Visited"><u>Visited</u></font></td>
</tr>
<tr>
<td bgcolor="$Accent" width="400"><font color="$AccentFont">$cn->{NAME}(Accent)</font> <font color="$Link"><u>Link</u></font> <font color="$Visited"><u>Visited</u></font></td>
<td bgcolor="$Background" width="400"><font color="white">$cn->{NAME}(Backgound)</font></td>
</tr>
END
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
    if (not $first) {
      print OUT <<END;
</table>
$copyright
</center>
</body>
</html>
END
      close OUT;
    }
  }
}
sub gendate {
  my($time) = @_;
  $tm = localtime($time);
  $year = $tm->year + 1900;
  $month = $tm->mon + 1;
  $day = $tm->mday;
  $hour = $tm->hour;
  $min = $tm->min;
  $sec = $tm->sec;
  return sprintf("$year/%02d/%02d $hour:$min:$sec", $month, $day);
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
