# File: HistogramAccessLog_Perl.pl
# Date            Author        number of nodes
# 2003/9/9        Walking Lint  81
# 2003/9/9        Walking Lint  80 (HistogramWebLog_Perl->HistogramAccessLog_Perl)
# 2003/8/13       Walking Lint  80
# 2003/7/14       Walking Lint  80
# 2003/3/14       Walking Lint  65
# 2003/3/3        Walking Lint  30
# 2002/11/27      Walking Lint  144 (HistogramLog_Perl->HistogramWebLog_Perl)
# 2002/10/22      Walking Lint  142
# 2002/7/4        Walking Lint  135 (AnalyzeLog_Perl->HistogramLog_Perl)
# 2002/4/4        Walking Lint  134
# 2002/3/3        Walking Lint  130
# 2002/3/2        Walking Lint  89
# 2002/2/17       Walking Lint  89
# 2002/2/16       Walking Lint  85

=pod

=head1 NAME

HistogramAccessLog_Perl.pl - generate histogram from web server's access log

=head1 DESCRIPTION

ex1. perl.exe HistogramAccessLog_Perl.pl "D:\tmp\access.log"

ex2. perl.exe HistogramAccessLog_Perl.pl "D:\tmp\access.log" 1 50

=head1 AUTHOR

Walking Lint

=cut

use File::Basename;
use Socket;
$graphlength = 30;
$from_file = shift; # log file
DebugPrint("from_file = $from_file");
$show_client = shift;
$convert = shift; # num of IP to convert ...
if ($show_client) {
  $use_log = 1; # this is assumed ...
} else {
  $use_log = shift;
}
%accesscount = ();
%ip2host_cache = ();
open IN, $from_file || die("Can't open $from_file: $!\n");
while (<IN>) {
  # customize here according to your log file format ... actually only $url (or if specified $client_ip) is used ...
  ($client_ip, $dummy, $dummy, $date, $date2, $kind, $url, $protocol, $port, $byte) = split(/ /, $_);
  if (($kind eq '"HEAD') or ($url =~ /\.(css|gif|jpg)$/)) { # better ???
    next;
  }
  if ($show_client) {
    $accesscount{$client_ip}++;
  } else {
    $accesscount{$url}++;
  }
}
close IN;
if ($use_log) {
  $log_file = dirname($from_file);
  $log_file =~ s/[\\]*$//;
  $log_file .= '\HistogramWebLog_Perl.log';
  DebugPrint("log_file = $log_file");
  open LOG, ">$log_file" or die("Cannot open $log_file: $!\n");
  select LOG;
  $| = 1; # no buffering ...
}
$max = 0;
foreach $key (keys %accesscount) {
  $tmp = $accesscount{$key};
  if ($tmp > $max) {
    $max = $tmp;
  }
}
$elemcount = $max / $graphlength;
foreach $key (sort {$accesscount{$b} <=> $accesscount{$a}} keys %accesscount) {
  $count = $accesscount{$key};
  $graphline = '*' x (int($accesscount{$key} / $elemcount));
  if ($use_log) { # actually $show_client ...
    if ($convert > 0) { # not functioning as expected ^^;
      $convert--;
      $org_key = $key;
      DebugPrint("org_key = " . $org_key);
      $key = gethostbyaddr(inet_aton($key), AF_INET) or next; # die "Cannot resolve $key: $!\n"
      DebugPrint("key = " . $key);
      format LOG =
@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> @>>>>>>>>>>>>>>>> - @>>>>>>>> @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$key $org_key $count $graphline
.
    } else {
      format LOG =
@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> - @>>>>>>>> @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$key $count $graphline
.
    }
  } else {
    format STDOUT =
@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> - @>>>>>>>> @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$key $count $graphline
.
  }
  write;
}
if ($use_log) {
  close LOG;
  DebugPrint("\ngenerated " . $log_file);
} else {
  print "\n";
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
