# File: HistogramHistory_Perl.pl
# Date            Author        number of nodes
# 2004/6/22       Walking Lint  111
# 2003/5/14       Walking Lint  104
# 2003/5/14       Walking Lint  104 (HistogramHistoryWithTk_Perl->HistogramHistory_Perl)
# 2003/4/8        Walking Lint  104
# 2002/12/23      Walking Lint  104 (HistogramHistory2_Perl->HistogramHistoryWithTk_Perl)
# 2002/12/23      Walking Lint  57 (HistogramHistory_Perl->HistogramHistory2_Perl)
# 2002/11/7       Walking Lint  57
# 2002/11/7       Walking Lint  57 (HistogramHistory_Perl->HistogramHistory_Perl)
# 2002/10/22      Walking Lint  142
# 2002/7/4        Walking Lint  135 (AnalyzeLog_Perl->HistogramLog_Perl)
# 2002/4/4        Walking Lint  134
# 2002/3/3        Walking Lint  130
# 2002/3/2        Walking Lint  89
# 2002/2/17       Walking Lint  89
# 2002/2/16       Walking Lint  85

=pod

=head1 NAME

HistogramHistory_Perl.pl - generate histogram from function point log

=head1 DESCRIPTION

ex. perl.exe HistogramHistory_Perl.pl HistogramHistoryWithTk_Perl.pat

=head1 AUTHOR

Walking Lint

=cut

$from_file = shift;
DebugPrint("from_file = $from_file");
$use_tk = shift;
DebugPrint("use_tk = " . $use_tk);
if (defined($use_tk)) {
  use Tk;
  use File::Basename;
  $width = 300;
  $max_height = 600;
  $elem_height = 12;
  $bottom_margin = 30;
  $left_margin = 80;
  $label_off = 10;
  $right_margin = 20;
  $top_margin = 50;
  $relative_height = 0.5;
} else {
  $graph_length = 30;
}
open IN, $from_file or die("Cannot open $from_file: $!\n");
$line = <IN>;
close IN;
@lines = split(/ \CRLF /, $line);
shift @lines;
shift @lines;
$latest_num_of_nodes = 0;
$first = 1;
foreach $historyitem (@lines) {
  if (length($historyitem) == 0) {
    next;
  }
  $historyitem =~ /^([0-9]+)\/([0-9]+)\/([0-9]+)/;
  $key = sprintf("%04d/%02d/%02d", $1, $2, $3);
  if ($historyitem =~ / \(/) { # this is assumed
    $historyitem =~ / \(([^\)]+)\)$/;
    $change{$key} = $1;
    $historyitem =~ s/ \(([^\)]+)\)$//;
  }
  $historyitem =~ /([0-9]+)$/;
  if (not defined($history{$key})) { # this is needed ...
    $history{$key} = $1;
  }
  if ($first) {
    $first = 0;
    $latest_num_of_nodes = $1;
  }
}
$max = 0;
$count = 0;
foreach $key (keys %history) {
  $tmp = $history{$key};
  if ($tmp > $max) {
    $max = $tmp;
  }
  $count++;
}
$title = basename($from_file);
$title =~ s/\.pat$//;
if (defined($use_tk)) {
  $mw = MainWindow->new;
  $mw->title("History");
  $height = $elem_height * $count + $top_margin + $bottom_margin;
  if ($height < $max_height) {
    $can = $mw->Canvas(-width => $width, -height => $height, -background => "white")
      ->pack();
  } else {
    $yscrl = $mw->Scrollbar()
      ->pack(-side => 'right', -fill => 'y');
    $can = $mw->Canvas(-width => $width, -height => $max_height, -background => "white", -scrollregion => [0, 0, $width, $height], -yscrollcommand =>['set', $yscrl])
      ->pack();
    $yscrl->configure(-command => ['yview', $can]);
  }
  $xoff = $left_margin;
  $yoff = $top_margin;
  $elem_width = ($width - $left_margin - $right_margin) / $max;
  DebugPrint("elem_width = $elem_width");
  $can->createText($width / 2, $top_margin / 3, -text => $title, -anchor => 'c');
  $can->createText($width / 2, $top_margin * 2 / 3, -text => "current num of nodes = " . $latest_num_of_nodes, -anchor => 'c');
} else {
  $elem_num = $max / $graph_length;
}
foreach $key (reverse sort keys %history) {
  if ($use_tk) {
    $can->createText($label_off, $yoff, -text => "$key", -anchor => 'nw');
    $can->createRectangle($xoff, $yoff + $elem_height * ((1 - $relative_height) / 2), $xoff + $elem_width * $history{$key}, $yoff + $elem_height * ((1 + $relative_height) / 2), -fill => 'green');
    $yoff += $elem_height;
  } else {
    $graphline = '*' x (int($history{$key} / $elem_num));
    format STDOUT =
@<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$key $graphline $change{$key}
.
    write;
  }
}
if ($use_tk) {
  MainLoop();
} else {
  printf "\n* = %d\n", $elem_num;
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
