# File: Links2Html_Perl.pl
# Date            Author        number of nodes
# 2003/8/21       Walking@Lint 268

=pod

=head1 NAME

Links2Html_Perl.pl - generate html from links

=head1 DESCRIPTION

ex. system () // '"' . $perl . '" "' . $templatesfolder . '\Links2Html_Perl.pl" "' . $link . '" "' . $weblogfolder . '\Link.html" "Weblogs"'

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use Jcode;
use File::Basename;
use Time::localtime;
use File::Copy;
$from_file = shift;
DebugPrint("from_file = $from_file");
$to_file = shift;
DebugPrint("to_file = $to_file");
$Title = shift;
DebugPrint("Title = $Title");
$bak_file = $from_file;
$bak_file =~ s/\.pat$/.bak/;
DebugPrint("bak_file = $bak_file");
$Copyright = shift;
$dir = &mydirname($to_file);
$copyright_file = $dir . '\copyright.html';
DebugPrint("copyright_file = $copyright_file");
if ((length($Copyright) == 0) and (-e $copyright_file)) {
  open IN, $copyright_file || die("Cannot open $copyright_file: $!\n");
  @copyrights = <IN>;
  close IN;
  $Copyright = join('', @copyrights);
}
$prev_path = shift;
DebugPrint("prev_path = $prev_path");
$link_file_body = 'link.css';
$align = shift;
$link_file = $to_file;
$link_file =~ s/[^\\]+$/$link_file_body/;
DebugPrint("link_file = $link_file");
$templatesfolder = $ENV{TEMPLATESFOLDER};
$templatesfolder =~ s/"//g;
$src_link_file = $templatesfolder . "\\" . $link_file_body;
if (not -e $link_file and -e $src_link_file) {
  copy($src_link_file, $link_file);
  DebugPrint("copied $src_link_file to $link_file");
}
my $root = {};
&initnode($root, 'root', '', '', '');
&gentree($root, $from_file);
if (defined($root->{CHILD})) {
  if (not defined($Title)) {
    $Title = $root->{CHILD}->{NAME};
  }
}
open OUT, ">$to_file";
print OUT <<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN">
<html>
<head>
<title>$Title</title>
<meta http-equiv="content-type" content="text/html;charset=Shift_JIS" />
<meta http-equiv="content-style-type" content="text/css" />
<link rel="stylesheet" type="text/css" href="$link_file_body" />
</head>
<body>
<table class="navi"><tr><td class="path">$prev_path$Title</td></tr></table>
END
$date = &gendate(time);
if (-e $bak_file) {
  @stat = stat($bak_file);
  $since = ' (since ' . &gendate($stat[9]) . ')';
}
print OUT <<END;
<div class="lastupdate">Last update: $date$since<br/>User-agent: UpdateStatusOfLinks/0.1</div>
END
&genhtml($root, 1);
print OUT <<END;
$Copyright
</body>
</html>
END
close OUT;
DebugPrint("generated $to_file");
sub gendate {
  my($time) = @_;
  $tm = localtime($time);
  $year = $tm->year + 1900;
  $month = $tm->mon + 1;
  $day = $tm->mday;
  $hour = $tm->hour;
  $min = $tm->min;
  $sec = $tm->sec;
  return sprintf("$year/%02d/%02d $hour:$min:$sec", $month, $day);
}
sub genhtml {
  my($nd, $level) = @_;
  if (defined($nd->{CHILD})) {
    my $cn = $nd->{CHILD};
    while (1) {
      $name = $cn->{NAME};
      if (not $name =~ /^_/) {
        $added = '';
        $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg; # [[CGI-Perl̊bu(p04)!http://www.parkcity.ne.jp/~chaichan/src/perl04.htm]]
        if ($cn->{TEXT} =~ /\.doc$/i) {
          $added = ' (Word)';
        } elsif ($cn->{TEXT} =~ /\.pdf$/i) {
          $added = ' (PDF)';
        } elsif ($cn->{TEXT} =~ /\.ppt$/i) {
          $added = ' (PowerPoint)';
        } elsif ($cn->{TEXT} =~ /\.xls$/i) {
          $added = ' (Excel)';
        }
        $type = $cn->{TYPE};
        $comment = '';
        if (length($type) != 0) {
          $rsslink = '';
          if ($type =~ /(http:[^ \(]*)/) { # not actually ... but this is assumed ...
            $rsslink = ' <a href="' . $1 . '"><img src="tinyxml.jpg" alt="RSS link" title="RSS link"></a>';
            $type = $cn->{TYPE};
            $type =~ s/(http:[^ \(]*)//;
          }
          if (length($type) != 0) {
            $type = $rsslink . ' - ' . $type;
          } else {
            $type = $rsslink;
          }
          $comment = $type;
        }
        if (length($comment) != 0) {
          $added .= $comment;
        }
        if (not ($cn->{KIND} =~ /\!/ and $from_file =~ /Link[0-9]*\.pat$/)) {
          my $tag = "h$level";
          my $attr = 'class="link"';
          if ($level > 6) {
            $tag = 'div';
            $attr = 'class="h' . $level . '"';
          }
          if ($cn->{KIND} =~ /\!/) {
            $added .= ' New!';
          }
          if ($cn->{KIND} =~ /\+/) {
            print OUT <<END;
<$tag $attr><img src="ExcludedNode.jpg" /> <a href="$cn->{TEXT}">$name</a>$added</$tag>
END
          } elsif ($cn->{KIND} =~ /\-/) {
            print OUT <<END;
<$tag $attr><img src="NotCheckedNode.jpg" /> <a href="$cn->{TEXT}">$name</a>$added</$tag>
END
          } elsif (defined($align)) {
            print OUT <<END;
<$tag $attr>&nbsp;&nbsp;&nbsp;&nbsp;<a href="$cn->{TEXT}">$name</a>$added</$tag>
END
          } else {
            print OUT <<END;
<$tag $attr><a href="$cn->{TEXT}">$name</a>$added</$tag>
END
          }
          &genhtml($cn, $level + 1);
        }
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
  }
}
sub mydirname {
  my($fn) = @_;
  my $res = &dirname($fn);
  $res =~ s/[\/\\]+$//;
  return $res;
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
sub Trace {
  my($str) = @_;
  print STDERR "$str\n";
}
