# File: MindMapWriter_Perl.pl
# Date            Author        number of nodes
# 2005/4/12       Walking Lint  233
# 2004/7/5        Walking Lint  228
# 2004/6/23       Walking Lint  218
# 2004/5/28       Walking Lint  205
# 2004/5/18       Walking Lint  196
# 2004/5/10       Walking Lint  167
# 2004/5/8        Walking Lint  146
# 2004/5/7        Walking Lint  143
# 2003/4/16       Walking Lint  126
# 2003/4/16       Walking Lint  107 (GenMindMap_Perl->MindMapWriter_Perl)
# 2003/4/14       Walking Lint  103
# 2002/11/16      Walking Lint  103
# 2002/10/10      Walking Lint  103
# 2002/10/2       Walking Lint  154
# 2002/10/1       Walking Lint  154
# 2002/9/29       Walking Lint  245
# 2002/9/28       Walking Lint  239
# 2002/9/21       Walking Lint  228
# 2002/9/21       Walking Lint  168 (GenMindMap_Perl->GenMindMap_Perl)
# 2002/8/31       Walking Lint  156

=pod

=head1 NAME

MindMapWriter_Perl.pl - generate .mm for FreeMind (http://freemind.sourceforge.net/index.html)

=head1 DESCRIPTION

activate as plug-in or filter

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use File::Basename;
$crlf = ' CR' . 'LF ';
@htmlelements = ('!doctype', 'a', 'center', 'div', 'embed', 'head', 'li', 'link', 'meta', 'ol', 'p', 'table', 'title', 'ul'); # actually embed can not be used ... see [[`lktv - I[vEuOER~jeB!http://ch.kitaguni.tv/docs/about/10tag.html]]
$include_type = shift; # try this ... now
DebugPrint("include_type = $include_type");
$include_text = shift; # try this ... now
DebugPrint("include_text = $include_text");
$include_text2 = shift; # try this ... now
DebugPrint("include_text2 = $include_text2");
$to_file = shift;
DebugPrint("to_file = $to_file");
$from_file = $to_file;
$from_file =~ s/\..+/.pat/;
my $root = {};
&initnode($root, 'root', '', '', '');
&gentree($root, $from_file);
$name2icon_file = mydirname($to_file) . '\\name2icon.pat';
unless (-e $name2icon_file) {
  $name2icon_file = '';
}
if (-e $name2icon_file) {
  my $root2 = {};
  &initnode($root2, 'root', '', '', '');
  &gentree($root2, $name2icon_file);
  if (defined($root2->{CHILD})) {
    $cn2 = $root2->{CHILD};
    %name2icon = ();
    while (1) {
      $name2icon{$cn2->{NAME}} = $cn2->{TEXT};
      DebugPrint("name2icon{$cn2->{NAME}} = $name2icon{$cn2->{NAME}}");
      if (not defined($cn2->{NEXT})) {
        last;
      }
      $cn2 = $cn2->{NEXT};
    }
  }
}
# customize here if needed ...
if (length($to_file) != 0) {
  open OUT, ">$to_file";
}
if (not &check_if_only_one_valid_node_exists($root)) {
  $title = basename($to_file);
  $title =~ s/\..+//;
  $addednode_start = '<node TEXT="' . $title . '">';
  $addednode_end = '</node>';
} else {
  $addednode_start = '';
  $addednode_end = '';
}
if (length($to_file) != 0) {
  print OUT <<END;
<?xml version="1.0" encoding="Shift_JIS" ?>
<map>$addednode_start
END
} else {
  print <<END;
<?xml version="1.0" encoding="Shift_JIS" ?>
<map>$addednode_start
END
}
&genmindmap($root, 0);
if (length($to_file) != 0) {
  print OUT <<END;
$addednode_end</map>
END
  close OUT;
} else {
  print <<END;
$addednode_end</map>
END
}
DebugPrint("generated $to_file");
sub check_if_only_one_valid_node_exists {
  my($nd) = @_;
  $cn = $nd->{CHILD};
  my $count = 0;
  if (defined($cn)) {
    while (1) {
      if ($cn->{NAME} =~ /^_/) {
      } else {
        $count++;
        if ($count > 1) {
          return 0;
        }
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
  }
  return 1;
}
sub genmindmap {
  my($nd, $level) = @_;
  if (defined($nd->{CHILD})) {
    my $first = 1;
    my $cn = $nd->{CHILD};
    while (1) {
      $type = '';
      $text = '';
      if (($cn->{NAME} =~ /^_/) and not ($cn->{TEXT} =~ /title=/) or ($cn->{NAME} eq 'center') or ($cn->{NAME} eq 'l') or ($cn->{NAME} eq 'p')) { # think later ...
        # skip
      } elsif ((length($cn->{NAME}) == 0) or &ishtmlelement($cn->{NAME})) {
        genmindmap($cn, $level);
      } else {
        my $indent = "\t" x ($level + 1);
        my $name = $cn->{NAME};
        if ($include_type) {
          if (length($cn->{TYPE})) {
            $type = $cn->{TYPE};
            $name .= ' ' . $type;
          }
        }
        if ($include_text) {
          if (length($cn->{TEXT})) {
            $text = $cn->{TEXT};
            $text =~ s/$crlf/ /g;
            $name .= ' ' . $text;
          }
        } elsif ($include_text2) {
          if (defined($cn->{NEXT}) and ($cn->{NEXT}->{NAME} =~ /^_text$/) and (length($cn->{NEXT}->{TEXT}))) {
            $text = $cn->{NEXT}->{TEXT};
            $text =~ s/$crlf/ /g;
            $name .= ' ' . $text;
          }
        }
        my $link = '';
        if ($name eq 'a') {
          $name = $cn->{TEXT};
          $link = $cn->{TYPE};
          $link =~ /href="([^"]*)"/;
          $link = $1;
        } elsif ($name =~ /^<a /) {
          $name =~ /<a href="([^"]*)">([^<]*)<\/a>/;
          $link = $1;
          $name = $2;
          if (length($type) != 0) { # try this ...
            $name = $name . ' ' . $type;
          }
          if (length($text) != 0) { # try this ...
            $name = $name . ' ' . $text;
          }
        } elsif (($name =~ /^_/) and ($cn->{TEXT} =~ /title="([^"]*)"/)) {
          $name = $1;
          $link = $cn->{TYPE};
        } elsif ($cn->{KIND} =~ /\//) {
          if (length($cn->{TYPE}) != 0) {
            $link = $cn->{TYPE} . '.mm';
          } else {
            $link = $cn->{NAME} . '.mm';
            $link =~ s/^[ ]+//;
            $link =~ s/\// /g;
          }
        } elsif (defined($cn->{CHILD}) and ($cn->{CHILD}->{KIND} =~ /\//) and ($cn->{CHILD}->{NAME} =~ /^http/)) { # think later ...
          $link = $cn->{CHILD}->{NAME};
        } else {
          if ($cn->{TEXT} =~ /^http/) {
            $link = $cn->{TEXT};
          }
        }
        if (length($link) != 0) {
          $link =~ s/&/\&amp;/;
          $link = 'LINK="' . $link . '" ';
        }
        $name =~ s/"/ /g;
        $name =~ s/\x8c$//g; # hack...
        $name =~ s/<[^>]+>//g;
        $name =~ s/>/\&gt;/g;
        $name =~ s/</\&lt;/g;
        $name =~ s/&/\&amp;/g; # actually not sure ...
        $icon = '';
        if ($cn->{KIND} =~ /\+/) {
          $icon = '<icon BUILTIN="button_ok"/>';
        } elsif ($cn->{KIND} =~ /\-/) {
          $icon = '<icon BUILTIN="messagebox_warning"/>'; # try this ...
        }
        # customize here ... icon for each person
        foreach $key (keys %name2icon) {
          if ($cn->{TEXT} =~ /$key/) {
            $icon .= '<icon BUILTIN="' . $name2icon{$key} . '"/>';
          }
        }
        if (not defined($cn->{CHILD}) or ($cn->{CHILD}->{KIND} =~ /\//) and ($cn->{CHILD}->{NAME} =~ /^http/)) { # currently ...
          if (length($to_file) != 0) {
            print OUT <<END;
${indent}<node ${link}TEXT="${name}">$icon</node>
END
          } else {
            print <<END;
${indent}<node ${link}TEXT="${name}">$icon</node>
END
          }
        } else {
          if (length($to_file) != 0) {
            print OUT <<END;
${indent}<node ${link}TEXT="${name}">$icon
END
          } else {
            print <<END;
${indent}<node ${link}TEXT="${name}">$icon
END
          }
          genmindmap($cn, $level + 1);
          if (length($to_file) != 0) {
            print OUT <<END;
${indent}</node>
END
          } else {
            print <<END;
${indent}</node>
END
          }
        }
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
  }
}
sub ishtmlelement {
  my($str) = @_;
  foreach $htmlelement (@htmlelements) {
    $str =~ tr/A-Z/a-z/;
    if ($str eq $htmlelement) {
      return 1;
    }
  }
  return 0;
}
sub mydirname {
  my($fn) = @_;
  my $res = &dirname($fn);
  $res =~ s/[\/\\]+$//;
  return $res;
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
