# File: OpmlWriter_Perl.pl
# Date            Author        number of nodes
# 2004/6/28       Walking Lint  125
# 2003/3/6        Walking Lint  126
# 2003/3/3        Walking Lint  120
# 2003/2/13       Walking Lint  219 (GenRSS_Perl->OpmlWriter_Perl)
# 2002/11/25      Walking Lint  215
# 2002/11/25      Walking Lint  212 (GenRSS_Perl->NewFile)
# 2002/11/18      Walking Lint  212
# 2002/11/15      Walking Lint  212
# 2002/11/5       Walking Lint  212
# 2002/11/1       Walking Lint  205
# 2002/10/31      Walking Lint  184
# 2002/10/28      Walking Lint  161
# 2002/10/22      Walking Lint  138
# 2002/10/18      Walking Lint  136
# 2002/10/17      Walking Lint  76 (GenRSS_Perl->GenRSS_Perl)
# 2002/10/11      Walking Lint  178 (GenCategorizedLinkLog_Perl->GenRSS_Perl)
# 2002/10/10      Walking Lint  178
# 2002/10/10      Walking Lint  172 (GenCategorizedLinkLogNew_Perl->GenCategorizedLinkLog_Perl)
# 2002/9/21       Walking Lint  109 (GenCategorizedLinkLog_Perl->GenCategorizedLinkLogNew_Perl)
# 2002/9/3        Walking Lint  149
# 2002/7/4        Walking Lint  141
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

OpmlWriter_Perl.pl - OPML writer

=head1 DESCRIPTION

astivate as plug-in ex. .opml () // OpmlReader_Python %% OpmlWriter_Perl

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use Jcode;
use File::Basename;
use Time::localtime;
$to_file = shift;
$from_file = $to_file;
$from_file =~ s/\..+/.pat/;
@htmlelements = ('a', 'center', 'div', 'li', 'ol', 'p', 'table', 'ul');
@skippableelements = ('!doctype', 'head');
my $root = {};
&initnode($root, 'root', '', '', '');
&gentree($root, $from_file);
open OUT, ">$to_file";
$title = basename($to_file);
$lines = <<END;
<?xml version="1.0"?>
<opml version="1.1">
<head>
<title>$title</title>
</head>
<body>
END
Jcode::convert(\$lines, 'utf8', 'sjis', "z");
print OUT $lines;
&genopml($root);
$lines = <<END;
</body>
</opml>
END
Jcode::convert(\$lines, 'utf8', 'sjis', "z");
print OUT $lines;
close OUT;
# may be better to convert URL ref [[title!URL]] pattern
sub escapetext {
  my($text) = @_;
  $text =~ s/&/\&amp;/;
  $text =~ s/>/\&gt;/;
  $text =~ s/</\&lt;/;
  return $text;
}
sub genopml {
  my($nd) = @_;
  if (defined($nd->{CHILD})) {
    my $cn = $nd->{CHILD};
    while (1) {
      if (defined($cn->{CHILD})) {
        if (&isskippableelement($cn->{NAME})) {
        } elsif (&ishtmlelement($cn->{NAME})) {
          &genopml($cn);
        } else {
          $lines = &gentextfromnode($cn, 1);
          Jcode::convert(\$lines, 'utf8', 'sjis', "z");
          print OUT $lines;
          &genopml($cn);
          print OUT '</outline>' . "\n";
        }
      } else {
        if (&isskippableelement($cn->{NAME}) or &ishtmlelement($cn->{NAME})) {
        } else {
          $lines = &gentextfromnode($cn, 0);
          Jcode::convert(\$lines, 'utf8', 'sjis', "z");
          print OUT $lines;
        }
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
  }
}
sub gentextfromnode {
  my($nd, $is_parent) = @_;
  $name = $nd->{NAME};
  if ($name eq '_text') {
    $text = &escapetext($nd->{TEXT}); # should escape " etc... think later...
    $text =~ s/ \CRLF / /g; # this is hack ...
    @texts = split(/ ----- /, $text);
    return '<outline text="' . join('"/>' . "\n" . '<outline text="', @texts) . '"/>' . "\n";
  } elsif (length($nd->{TEXT}) == 0) {
    if ($is_parent) {
      return '<outline text="' . $nd->{NAME} . '">' . "\n";
    } else {
      return '<outline text="' . $nd->{NAME} . '"/>' . "\n";
    }
  } else {
    $text = &escapetext($nd->{TEXT}); # should escape " etc... think later...
    $text =~ s/ \CRLF / /g; # this is hack ...
    @texts = split(/ ----- /, $text);
    if ($is_parent) {
      return '<outline text="' . $nd->{NAME} . '">' . "\n" . '<outline text="' . join('"/>' . "\n" . '<outline text="', @texts) . '"/>' . "\n";
    } else {
      return '<outline text="' . $nd->{NAME} . '">' . "\n" . '<outline text="' . join('"/>' . "\n" . '<outline text="', @texts) . '"/>' . "\n" . '</outline>' . "\n";
    }
  }
}
sub ishtmlelement {
  my($str) = @_;
  $str =~ tr/A-Z/a-z/;
  if (($str =~ /^_/) and not ($str eq '_text')) {
    return 0;
  }
  foreach $htmlelement (@htmlelements) {
    if (($str eq $htmlelement) or ($str eq '<' . $htmlelement . '>')) {
      return 1;
    }
  }
  return 0;
}
sub isskippableelement {
  my($str) = @_;
  $str =~ tr/A-Z/a-z/;
  if (($str =~ /^_/) and not ($str eq '_text')) {
    return 1;
  }
  foreach $skippableelement (@skippableelements) {
    if (($str eq $skippableelement) or ($str eq '<' . $skippableelement . '>')) {
      return 1;
    }
  }
  return 0;
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
sub Trace {
  my($str) = @_;
  print STDERR "$str\n";
}
