# File: PatentClassReader_Perl.pl
# Date            Author        number of nodes
# 2006/3/29       Walking Lint  203
# 2006/3/22       Walking Lint  201
# 2005/4/4        Walking Lint  176
# 2005/3/29       Walking Lint  176
# 2005/3/25       Walking Lint  162
# 2005/3/24       Walking Lint  158
# 2005/3/23       Walking Lint  140
# 2005/3/17       Walking Lint  138
# 2005/3/8        Walking Lint  124
# 2005/3/7        Walking Lint  72
# 2005/3/7        Walking Lint  62 (FTermTableReader_Perl->PatentClassReader_Perl)
# 2005/2/10       Walking Lint  62

=pod

=head1 NAME

PatentClassReader_Perl.pl - patent class reader

=head1 DESCRIPTION

activate as filter

<a href="http://www.uspto.gov/go/classification/">Patent Classification Home Page - Internet</a>

ex. <a href="http://www.uspto.gov/go/classification/uspc704/sched704.htm">Class Schedule for Class 704 DATA PROCESSING: SPEECH SIGNAL PROCESSING, LINGUISTICS, LANGUAGE TRANSLATION, AND AUDIO COMPRESSION/DECOMPRESSION</a>

=head1 AUTHOR

Walking Lint

=cut

use CVTree_Perl;
$pcnums = shift;
DebugPrint("pcnums = $pcnums");
$to_dir = shift;
DebugPrint("to_dir = $to_dir");
if ((length($to_dir) == 0) and (length($pcnums) != 0)) {
  $to_dir = $pcnums;
  undef $pcnums;
}
$list_fn = $ENV{TEMPLATESFOLDER} . "\\" . "PatentClassList.txt";
%list = ();
if (-e $list_fn) {
  open IN, $list_fn or die("Cannot open $list_fn: $!\n");
  while (<IN>) {
    $_ =~ s/[\r\n]+$//;
    ($name, $def) = split(/\t/, $_);
    $list{$name} = $def;
  }
  close IN;
}
if (length($pcnums) == 0) {
  &readtable('');
} else {
  @pcnums = split(/\t/, $pcnums);
  foreach $pcnum (@pcnums) {
    &readtable($pcnum);
  }
}
open OUT, ">$list_fn" or die("Cannot open $list_fn: $!\n"); # PatentClassList.txt is updated ...
foreach $key (sort keys %list) {
  print OUT "$key\t$list{$key}\n";
}
close OUT;
sub readtable {
  my($pcnum) = @_;
  if (length($pcnum) == 2) {
    $pcnum = '0' . $pcnum;
  } elsif (length($pcnum) == 1) {
    $pcnum = '00' . $pcnum;
  }
  if (length($pcnum) == 0) {
    @lines = <STDIN>;
  } else {
    use LWP::UserAgent;
    use Win32::Registry;
    use Win32::Clipboard;
    use Jcode;
    $HKEY_CURRENT_USER->Open("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings", $ref) or die "cannot open registry: $!";
    $ref->GetValues(\%values);
    $refproxyoverride = $values{'ProxyOverride'};
    $refproxyserver = $values{'ProxyServer'};
    $ref->Close();
    $proxyoverride = $$refproxyoverride[2];
    $proxyoverride =~ s/\*//g;
    $proxyoverride =~ s/;?<local>//g;
    $proxyoverride =~ s/;/\|/g; # not actually ...
    $proxyoverride = quotemeta $proxyoverride;
    $proxyoverride =~ s/\\\|/\|/g; # hack ...
    $proxyoverride = '(' . $proxyoverride . ')';
    $proxyserver = $$refproxyserver[2];
    $url = 'http://www.uspto.gov/go/classification/uspc' . $pcnum . '/sched' . $pcnum . '.htm';
    $webagent = new LWP::UserAgent;
    unless ($url =~ /$proxyoverride/) {
      $webagent->proxy(['ftp', 'http'] => "http://$proxyserver");
    }
    $request = HTTP::Request->new('GET' => $url);
    my $response = $webagent->request($request);
    if ($response->is_success) {
      $doc = $response->content;
      $doc =~ s/[\n\r]+/\n/g; # not sure if needed ...
      @lines = split(/\n/, $doc);
    } else {
      DebugPrint("Cannot download $url");
      exit;
    }
  }
  $list_fn = $ENV{TEMPLATESFOLDER} . "\\" . "PatentClassList.txt";
  @fterms = ();
  @ftermdefs = ();
  $foundtop = 0;
  $top = '';
  $topdef = '';
  foreach $line (@lines) {
    if (($foundtop == 0) and ($line =~ /">Class \&nbsp; ([^<]*)<\/a><\/td><td valign="top" class="ClassTitle">([^<]*)/)) {
      $top = $1;
      $foundtop = 1;
      $topdef = $2;
    } elsif ($line =~ />([0-9\.A-Z]+) <\/a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<\/td><td  width="85%" class="SubTtl"><b><big><\/big>([^<]+)<\/b>/) {
      push @fterms, $1;
      push @ftermlevels, 1;
      $ftermdef = $2;
      $ftermdef =~ s/^[ ]+//;
      $ftermdef =~ s/[ ]+$//;
      push @ftermdefs, $ftermdef;
    } elsif ($line =~ /Subclass ([0-9\.A-Z]+) indent level is ([0-9]+)"><\/big>([^<]+)<\/td>/) {
      push @fterms, $1;
      push @ftermlevels, $2 + 1;
      $ftermdef = $3;
      $ftermdef =~ s/^[ ]+//;
      $ftermdef =~ s/[ ]+$//;
      push @ftermdefs, $ftermdef;
    } elsif ($line =~ /Subclass   indent level is ([0-9]+)"><\/big>([^<]+)<\/td>/) { # hack
      push @fterms, ' ';
      push @ftermlevels, $1 + 1;
      $ftermdef = $2;
      $ftermdef =~ s/^[ ]+//;
      $ftermdef =~ s/[ ]+$//;
      push @ftermdefs, $ftermdef;
    }
  }
  if ((length($pcnum) != 0) and (length($top) == 0)) {
    DebugPrint("$pcnum ͂܂");
    exit;
  }
  if (length($topdef) == 0) {
    $topdef = 'none';
  }
  if (&updatelist($top, $topdef) == 0) { # not obsolete
    my $root = {};
    &initnode($root, 'root', '', '', '');
    $node = &addchildnode($root, $top, '', '', $topdef);
    &gentree($node);
    $to_file = '';
    if (length($to_dir) != 0) {
      $to_file = $to_dir . "\\" . $top . ".pat";
    }
    &savetree($root, $to_file);
    if (length($to_dir) != 0) {
      &savetree($root, ''); # hack ...
      DebugPrint("generated $to_file");
    }
  } else {
    print "$top - obsolute?\n";
    DebugPrint("$top - obsolute?");
  }
}
sub gentree {
  my($nd) = @_;
  $prevlevel = -1;
  @prevlevels = ();
  push @prevlevels, $prevlevel; # not sure if needed ...
  my $cur = $nd;
  foreach $fterm (@fterms) {
    $name = $fterm;
    $text = shift @ftermdefs;
    $level = shift @ftermlevels;
    # construct tree here ...
    if ($prevlevel < $level) {
      push @prevlevels, $prevlevel;
      $cur = addchildnode($cur, $name, $kind, $type, $text);
    } elsif ($prevlevel == $level) {
      $cur = addnode($cur, $name, $kind, $type, $text);
    } else {
      until ($prevlevel <= $level) {
        if ($cur->{PARENT} == $root) { # hack ???
          last;
        }
        $cur = $cur->{PARENT};
        $prevlevel = pop @prevlevels;
      }
      $cur = addnode($cur, $name, $kind, $type, $text);
    }
    $prevlevel = $level;
  }
}
sub updatelist {
  my($nm, $df) = @_;
  $obsolete = 0;
  if (($df eq 'none') and defined($list{$nm}) and not ($list{$nm} eq 'none')) {
    $df = $list{$nm} . ' - obsolete?';
    $obsolete = 1;
  }
  $list{$nm} = $df;
  return $obsolete;
}
sub DebugPrint {
  my($str) = @_;
  print STDERR "$str\n";
}
