# File: ReSchedule_Perl.pl
# Date            Author        number of nodes
# 2002/12/26      Walking Lint  153
# 2002/12/5       Walking Lint  146
# 2002/12/1       Walking Lint  48 (ReCalc_Perl->ReSchedule_Perl)
# 2002/11/29      Walking Lint  93 (GeppouReader_Perl->ReCalc_Perl)
# 2002/11/24      Walking Lint  315 (PythonReader_Perl->GeppouReader_Perl)
# 2002/11/24      Walking Lint  287 (RubyReader_Perl->PythonReader_Perl)
# 2002/11/23      Walking Lint  173
# 2002/11/13      Walking Lint  240 (CVTree_Perl->RubyReader_Perl)
# 2002/11/10      Walking Lint  245
# 2002/11/8       Walking Lint  244
# 2002/11/6       Walking Lint  225
# 2002/11/1       Walking Lint  226
# 2002/10/23      Walking Lint  218 (CVTree->CVTree_Perl)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Perl->CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Perl->ExtractCategorizedText_Perl)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Perl->GenCategorizedText_Perl)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

ReSchedule_Perl.pl - re-schedule

=head1 DESCRIPTION

ex. perl.exe ReSchedule_Perl.pl schedule2003.pat

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use Time::localtime;
use Time::Local;
@yobi = ('', '', '', '', '', '', 'y');
$from_file = shift;
DebugPrint("from_file = $from_file");
$to_file = $from_file;
my $root = {};
&initnode($root, 'root');
&gentree($root, $from_file);
&recalc($root);
&reschedule($root, '');
&savetree($root, $to_file);
DebugPrint("modified $to_file");
sub add_delta_days {
  my($date, $days) = @_;
  ($year, $month, $day) = split(/[\/ \(]/, $date);
  $time = timelocal(0, 0, 0, $day, $month - 1, $year - 1900);
  DebugPrint("time = $time");
  $count = $days;
  while ($count > 0) {
    $time += 60 * 60 * 24;
    $tm = localtime($time);
    $lastyear = $tm->year + 1900;
    $lastmonth = $tm->mon + 1;
    $lastday = $tm->mday;
    if (not &ishoriday($lastyear, $lastmonth, $lastday)) {
      $count--;
    }
  }
  my $lastdate = sprintf("%s/%02d/%02d", $lastyear, $lastmonth, $lastday);
  $lastdate .= '(' . $yobi[int($lastyear + int($lastyear / 4) - int($lastyear / 100) + int($lastyear / 400) + int((13 * $lastmonth + 8) / 5) + $lastday) % 7] . ')';
  return $lastdate;
}
# [[Re: Getting Day of Week for a specific date!http://granite.unh.edu/pasnews/msg00674.html]]
sub day_of_week {
  my($y, $m, $d) = @_;
  @t = (0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4);
  $y -= ($m < 3);
  return ($y + int($y / 4) - int($y / 100) + int($y / 400) +$t[$m - 1] + $d) % 7;
}
sub ishoriday {
  my($year, $month, $day) = @_;
  $res = 0;
  $yobi = &day_of_week($year, $month, $day);
  if ($yobi == 0 or $yobi == 6) {
    $res = 1;
  }
  # add here some more checks for other horidays ...
  return $res;
}
sub normalize {
  my($date) = @_;
  ($year, $month, $day) = split(/[\/ \(]/, $date);
  $date = sprintf("%s/%02d/%02d", $year, $month, $day);
  $date .= '(' . $yobi[&day_of_week($year, $month, $day)] . ')';
  return $date;
}
sub recalc {
  my($nd) = @_;
  my $val = 0;
  if (defined($nd->{CHILD})) {
    my $cn = $nd->{CHILD};
    while (1) {
      if (not $cn->{NAME} =~ /^_/) {
        $val += &recalc($cn);
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
    if (not ($nd->{NAME} eq 'root')) {
      $nd->{TYPE} = $val;
    }
  } else {
    $val = $nd->{TYPE};
    $val =~ s/^[ ]*//;
  }
  return $val;
}
sub reschedule {
  my($nd, $date) = @_;
  Trace("enter reschedule($nd->{NAME}, $date)");
  if (not $nd->{NAME} eq 'root') {
    if (length($date) == 0) {
      $text = $nd->{TEXT};
      if ($text =~ / - /) {
        ($date, $dummy) = split(/ - /, $text);
      } else {
        $date = $text;
      }
      $date = &normalize($date);
    }
  }
  my $nextdate = $date;
  my $lastdate = $nextdate;
  if (defined($nd->{CHILD})) {
    my $cn = $nd->{CHILD};
    while (1) {
      if (not $cn->{NAME} =~ /^_/) {
        if ($nd->{NAME} eq 'root') { # may be hack ... think later ...
          ($lastdate, $nextdate) = &reschedule($cn, '');
        } else {
          ($lastdate, $nextdate) = &reschedule($cn, $nextdate);
        }
      }
      if (not defined($cn->{NEXT})) {
        last;
      }
      $cn = $cn->{NEXT};
    }
  } else {
    if (length($date) != 0) {
      if ($nd->{TYPE} > 1) {
        $lastdate = &add_delta_days($date, $nd->{TYPE} - 1);
      }
      $nextdate = &add_delta_days($date, $nd->{TYPE});
    }
  }
  if (length($date) != 0) {
    if ((length($lastdate) != 0) and (not $date eq $lastdate)) {
      $nd->{TEXT} = $date . ' - ' . $lastdate;
    } else {
      $nd->{TEXT} = $date;
    }
  }
  DebugPrint("lastdate = $lastdate");
  DebugPrint("nextdate = $nextdate");
  return ($lastdate, $nextdate);
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
