# File: TextWithHierarchyReader_Perl.pl
# Date            Author        number of nodes
# 2004/7/21       Walking Lint  124
# 2004/4/15       Walking Lint  103
# 2003/10/3       Walking@Lint 106
# 2003/5/17       Walking Lint  98
# 2003/5/8        Walking Lint  92
# 2003/5/8        Walking Lint  170 (TextWithStructureReader_Perl->TextWithHierarchyReader_Perl)
# 2003/5/8        Walking Lint  170
# 2003/5/8        Walking Lint  315 (PythonReader_Perl->TextWithStructureReader_Perl)
# 2002/12/1       Walking Lint  315
# 2002/11/24      Walking Lint  315
# 2002/11/24      Walking Lint  287 (RubyReader_Perl->PythonReader_Perl)
# 2002/11/23      Walking Lint  173
# 2002/11/13      Walking Lint  240 (CVTree_Perl->RubyReader_Perl)
# 2002/11/10      Walking Lint  245
# 2002/11/8       Walking Lint  244
# 2002/11/6       Walking Lint  225
# 2002/11/1       Walking Lint  226
# 2002/10/23      Walking Lint  218 (CVTree->CVTree_Perl)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Perl->CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Perl->ExtractCategorizedText_Perl)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Perl->GenCategorizedText_Perl)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

TextWithHierarchyReader_Perl.pl - txt with hierarchy reader

=head1 DESCRIPTION

activate as plug-in or filter

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
use File::Basename;
use Time::localtime;
$from_file = shift;
DebugPrint("from_file = $from_file");
$to_file = $from_file;
$to_file =~ s/\..+/.pat/;
my $root = {};
$tm = localtime;
$basename = '';
if (length($from_file) != 0) {
  $basename = basename($from_file);
} else {
  $basename = 'NewFile.txt';
}
$file_comment = 'File: ' . $basename . ' CR' . 'LF Date            Author        number of nodes CR' . 'LF ';
$date = sprintf("%d/%d/%d", $tm->year + 1900, $tm->mon + 1, $tm->mday);
$file_comment .=  $date;
&initnode($root, 'root', '', '', $file_comment);
&gentree($root, $from_file);
&savetree($root, $to_file);
DebugPrint("generated $to_file");
sub gentree {
  my($nd, $from_file) = @_;
  # gen tree from fn ...
  $prevname = '';
  $previndentedcomment = '';
  $prevlevel = -1;
  @prevlevels = ();
  push @prevlevels, $prevlevel; # not sure if needed ...
  my $cur = $nd;
  $first = 1;
  if (length($from_file) != 0) {
    open IN, $from_file || die("Can't open $from_file: $!\n");
    $infilehandle = 'IN'; # see [[SAGE - Perl Practicum - Know All the Angles!http://www.usenix.org/publications/perl/perl02.html]]
  } else {
    $infilehandle = 'STDIN';
  }
  $name = '';
  $text = '';
  $firsttext = 1;
  while (<$infilehandle>) {
    if ((length($_) != 0)) {
      # should treat following text ... think ...
      $_ =~ s/[\n\r]+$//;
      $_ =~ /^(\.*)(.*)/;
      $level = length($1);
      DebugPrint("level = $level");
      $name = $2;
      DebugPrint("name = $name");
      # construct tree here ...
      if ($level != 0) {
        if (length($name) != 0) { # this is assumed ...
          if ($prevlevel < $level) {
            push @prevlevels, $prevlevel;
            $cur = addchildnode($cur, $name, '', '', '');
          } elsif ($prevlevel == $level) {
            $cur = addnode($cur, $name, '', '', '');
          } else {
            until ($prevlevel <= $level) {
              $cur = $cur->{PARENT};
              $prevlevel = pop @prevlevels;
            }
            $cur = addnode($cur, $name, $kind, $type, '');
          }
          $prevlevel = $level;
        }
        $firsttext = 1;
      } else {
        if ($firsttext) {
          $firsttext = 0;
          $cur->{TEXT} = $_;
        } else {
          $cur->{TEXT} .= ' CR' . 'LF ' . $_;
        }
      }
    }
  }
  if (length($from_file) != 0) {
    close IN;
  }
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
