# File: dir_Perl.pl
# Date            Author        number of nodes
# 2002/12/7       Walking Lint  143
# 2002/12/4       Walking Lint  141
# 2002/12/1       Walking Lint  93 (GeppouReader_Perl->dir_Perl)
# 2002/11/29      Walking Lint  93
# 2002/11/24      Walking Lint  315 (PythonReader_Perl->GeppouReader_Perl)
# 2002/11/24      Walking Lint  287 (RubyReader_Perl->PythonReader_Perl)
# 2002/11/23      Walking Lint  173
# 2002/11/13      Walking Lint  240 (CVTree_Perl->RubyReader_Perl)
# 2002/11/10      Walking Lint  245
# 2002/11/8       Walking Lint  244
# 2002/11/6       Walking Lint  225
# 2002/11/1       Walking Lint  226
# 2002/10/23      Walking Lint  218 (CVTree->CVTree_Perl)
# 2002/10/16      Walking Lint  155
# 2002/10/15      Walking Lint  144
# 2002/10/8       Walking Lint  137
# 2002/10/1       Walking Lint  135
# 2002/9/30       Walking Lint  128
# 2002/9/30       Walking Lint  182 (ExtractCategorizedText_Perl->CVTree)
# 2002/9/28       Walking Lint  182
# 2002/9/22       Walking Lint  178
# 2002/7/16       Walking Lint  143 (GenCategorizedText_Perl->ExtractCategorizedText_Perl)
# 2002/7/15       Walking Lint  126
# 2002/7/4        Walking Lint  141 (GenCategorizedLinkLog_Perl->GenCategorizedText_Perl)
# 2002/6/7        Walking Lint  136
# 2002/6/4        Walking Lint  131
# 2002/4/21       Walking Lint  120
# 2002/4/12       Walking Lint  112
# 2002/4/4        Walking Lint  85

=pod

=head1 NAME

dir_Perl.pl - dir

=head1 DESCRIPTION

ex. perl.exe dir_Perl.pl C: D:\tmp\test99.pat 1

=head1 AUTHOR

Walking Lint

=cut

use lib '.';
use CVTree_Perl;
$from_dir = shift;
$from_dir =~ s/[\\]+$//;
$from_dir .= "\\";
if (not -d $from_dir) {
  DebugPrint("$from_dir is not a folder");
  exit;
}
DebugPrint("from_dir = $from_dir");
$to_file = shift;
DebugPrint("to_file = $to_file");
$max_level = shift;
if (not defined($max_level)) {
  $max_level = 99;
}
DebugPrint("max_level = $max_level");
$level = 0;
$graph_length = 50; # **************************************************
my $root = {};
&initnode($root, 'root');
&gentree($root, $from_dir, $level);
if (defined($root->{CHILD})) {
  $elem_size = int($root->{CHILD}->{NAME} / $graph_length);
  if ($elem_size != 0) {
    &sortedsavetree($root, $to_file);
  }
}
DebugPrint("* = $elem_size");
if (defined($to_file)) {
  DebugPrint("generated $to_file");
}
sub gentree {
  my($nd, $from_dir, $level) = @_;
  my $cn = {};
  if (not $level > $max_level + 1) { # actually not sure ...
    $cn = &addchildnode($nd, $from_dir, ' () / ', '', '');
  }
  my $total_size = 0;
  opendir DIR, $from_dir or dir("Cannot open $from_dir: $!");
  my @files = grep !/^\.\.?$/, readdir DIR;
  my $file = '';
  foreach $file (@files) {
    my $file2 = $from_dir . $file;
    if (-d $file2) {
      $total_size += gentree($cn, $file2 . "\\", $level + 1);
    } else {
      my @stat = stat($file2);
      my $size = $stat[7];
      if (not $level > $max_level) {
        &addchildnode($cn, $size, '', '', $file);
      }
      $total_size += $size;
    }
  }
  closedir DIR;
  if (not $level > $max_level + 1) {
    $cn->{TEXT} = $cn->{NAME};
    $cn->{NAME} = $total_size;
  }
  return $total_size;
}
sub savenode {
  my($nd, $ind) = @_;
  if (length($nd->{KIND}) == 0) { # this is hack ... change CVTree for root ...
    $kind = ' ()';
  } else {
    $kind = $nd->{KIND};
  }
  $kind =~ s/[ ]+$//;
  $type = '';
  if ((length($nd->{TYPE}) != 0) or (length($nd->{TEXT}) != 0)) {
    $type = " \/\/ " . ('*' x int($nd->{NAME} / $elem_size));
  }
  $text = '';
  if (length($nd->{TEXT}) != 0) {
    $text = " \%\% " . $nd->{TEXT};
  }
  printf $outfilehandle "%s%s%s%s%s\n", $ind, $nd->{NAME}, $kind, $type, $text;
}
sub sortedsavetree {
  my($nd, $to_file) = @_;
  $sep = "\t";
  if (length($to_file) != 0) {
    open OUT, ">$to_file";
    $outfilehandle = 'OUT';
  } else {
    $outfilehandle = 'STDOUT';
  }
  &savenode($nd, '');
  if (defined($nd->{CHILD})) {
    &sortedsavetreebody($nd->{CHILD}, $sep, $sep);
  }
  if (length($to_file) != 0) {
    close OUT;
  }
}
sub sortedsavetreebody {
  my($nd, $ind, $sep) = @_;
  my %nodes = ();
  $nodes{$nd->{NAME}} = $nd;
  while (defined($nd->{NEXT})) {
    $nd = $nd->{NEXT};
    $nodes{$nd->{NAME}} = $nd;
  }
  foreach $key (sort {$b <=> $a} keys %nodes) {
    &savenode($nodes{$key}, $ind);
    if (defined($nodes{$key}->{CHILD})) {
      &sortedsavetreebody($nodes{$key}->{CHILD}, $ind . $sep, $sep);
    }
  }
}
sub DebugPrint {
  my($str) = @_;
  print STDERR $str . "\n";
}
sub Trace {
  my($str) = @_;
  print STDERR $str . "\n";
}
