//
// File: hilbert3d_LSL.lsl
// Date            Author        number of nodes
// 2007/9/29       walkinglint   231
// 2007/9/29       walkinglint   231 (hilbert3d->hilbert3d_LSL)
// 2007/9/23       walkinglint   231
// 2007/9/23       walkinglint   80 (hilbert->hilbert3d)
// 2007/9/18       walkinglint   80
// 2007/9/17       walkinglint   86
// 2007/9/13       walkinglint   73
// 2007/9/13
//
//
// Courbe de Hilbert 3d
// http://www.mathcurve.com/fractals/hilbert3d/hilbert3d.shtml
//
integer i;
integer level = 2;
float dist0 = 1.6;
float dist;
vector cur;
vector init_pos = <-0.6, 1.0, -0.6>;
// not actually hilbertD may be currect though
hilbertA(integer level) {
    if (level > 0) {
        lineRel(<0, 0, dist>);
        lineRel(<dist, 0, 0>);
        lineRel(<0, 0, -dist>);
        lineRel(<0, dist, 0>);
        lineRel(<0, 0, dist>);
        lineRel(<-dist, 0, 0>);
        lineRel(<0, 0, -dist>);
    }
}
hilbertB(integer level) {
    if (level > 0) {
        lineRel(<0, dist, 0>);
        lineRel(<0, 0, dist>);
        lineRel(<0, -dist, 0>);
        lineRel(<dist, 0, 0>);
        lineRel(<0, dist, 0>);
        lineRel(<0, 0, -dist>);
        lineRel(<0, -dist, 0>);
    }
}
hilbertC(integer level) {
    if (level > 0) {
        lineRel(<dist, 0, 0>);
        lineRel(<0, 0, -dist>);
        lineRel(<-dist, 0, 0>);
        lineRel(<0, -dist, 0>);
        lineRel(<dist, 0, 0>);
        lineRel(<0, 0, dist>);
        lineRel(<-dist, 0, 0>);
    }
}
hilbertD(integer level) {
    if (level > 0) {
        hilbertA(level - 1);
        lineRel(<0, dist, 0>);
        hilbertB(level - 1);
        lineRel(<dist, 0, 0>);
        hilbertD(level - 1);
        lineRel(<0, -dist, 0>);
        hilbertC(level - 1);
        lineRel(<0, 0, dist>);
        hilbertA(level - 1);
        lineRel(<0, dist, 0>);
        hilbertD(level - 1);
        lineRel(<-dist, 0, 0>);
        hilbertF(level - 1);
        lineRel(<0, -dist, 0>);
        hilbertG(level - 1);
    }
}
hilbertE(integer level) {
    if (level > 0) {
        lineRel(<dist, 0, 0>);
        lineRel(<0, 0, dist>);
        lineRel(<-dist, 0, 0>);
        lineRel(<0, dist, 0>);
        lineRel(<dist, 0, 0>);
        lineRel(<0, 0, -dist>);
        lineRel(<-dist, 0, 0>);
    }
}
hilbertF(integer level) {
    if (level > 0) {
        lineRel(<0, dist, 0>);
        lineRel(<0, 0, -dist>);
        lineRel(<0, -dist, 0>);
        lineRel(<-dist, 0, 0>);
        lineRel(<0, dist, 0>);
        lineRel(<0, 0, dist>);
        lineRel(<0, -dist, 0>);
    }
}
hilbertG(integer level) {
    if (level > 0) {
        lineRel(<0, 0, -dist>);
        lineRel(<dist, 0, 0>);
        lineRel(<0, 0, dist>);
        lineRel(<0, -dist, 0>);
        lineRel(<0, 0, -dist>);
        lineRel(<-dist, 0, 0>);
        lineRel(<0, 0, dist>);
    }
}
goToXY(vector new) {
    cur = new;
}
lineRel(vector delta) {
    llRezObject("phy_bar3d", llGetPos() + <cur.x, cur.z, cur.y> + <delta.x, delta.z, delta.y> / 2, ZERO_VECTOR, ZERO_ROTATION, (integer)(llFabs(delta.x * 10) * 10000) + (integer)(llFabs(delta.y * 10) * 100) + (integer)llFabs(delta.z * 10));
    llSleep(0.05);
    cur += delta;
}
default {
    state_entry() {
        llSetTimerEvent(59);
    }
    on_rez(integer param) {
        dist = dist0;
        for (i = level; i > 0; i--) {
            dist /= 2;
        }
        goToXY(init_pos);
        hilbertD(level);        // start recursion
    }
    timer() {
        goToXY(init_pos);
        hilbertD(level);        // start recursion
    }
}
