//
// File: sierpinski_LSL.lsl
// Date            Author        number of nodes
// 2007/9/29       walkinglint   107
// 2007/9/29       walkinglint   107 (sierpinski->sierpinski_LSL)
// 2007/9/24       walkinglint   107
// 2007/9/24       walkinglint   80 (hilbert->sierpinski)
// 2007/9/18       walkinglint   80
// 2007/9/17       walkinglint   86
// 2007/9/13       walkinglint   73
// 2007/9/13
//
// http://en.wikipedia.org/wiki/Sierpi%C5%84ski_curve
integer i;
integer level = 3;
float dist0 = 0.8;
float dist;
vector cur;
float x_pos = -1.6;
float y_pos = 1.0;
sierpA(integer level) {
    if (level > 0) {
        sierpA(level - 1);
        lineRel(dist, dist);
        sierpB(level - 1);
        lineRel(2 * dist, 0);
        sierpD(level - 1);
        lineRel(dist, -dist);
        sierpA(level - 1);
    }
}
sierpB(integer level) {
    if (level > 0) {
        sierpB(level - 1);
        lineRel(-dist, dist);
        sierpC(level - 1);
        lineRel(0, 2 * dist);
        sierpA(level - 1);
        lineRel(dist, dist);
        sierpB(level - 1);
    }
}
sierpC(integer level) {
    if (level > 0) {
        sierpC(level - 1);
        lineRel(-dist, -dist);
        sierpD(level - 1);
        lineRel(-2 * dist, 0);
        sierpB(level - 1);
        lineRel(-dist, dist);
        sierpC(level - 1);
    }
}
sierpD(integer level) {
    if (level > 0) {
        sierpD(level - 1);
        lineRel(dist, -dist);
        sierpA(level - 1);
        lineRel(0, -2 * dist);
        sierpC(level - 1);
        lineRel(-dist, -dist);
        sierpD(level - 1);
    }
}
goToXY(float x, float y) {
    cur.x = x;
    cur.y = y;
}
lineRel(float deltaX, float deltaY) {
    integer is_minus = 0;
    if (deltaX * deltaY < 0) {
        is_minus = 1;
    } else {
        is_minus = 0;
    }
    llRezObject("phy_sierp_bar", llGetPos() + <cur.x + deltaX / 2, 0.0, cur.y + deltaY / 2>, ZERO_VECTOR, ZERO_ROTATION, is_minus * 10000 + (integer)(llFabs(deltaX * 10) * 100) + (integer)llFabs(deltaY * 10));
    llSleep(0.01);
    cur.x += deltaX;
    cur.y += deltaY;
}
default {
    state_entry() {
        llSetTimerEvent(120);
    }
    on_rez(integer param) {
        dist = dist0;
        for (i = level; i > 0; i--) {
            dist /= 2;
        }
        goToXY(x_pos, y_pos);
        sierpA(level);  // start recursion
        lineRel(dist, dist);
        sierpB(level);  // start recursion
        lineRel(-dist, dist);
        sierpC(level);  // start recursion
        lineRel(-dist, -dist);
        sierpD(level);  // start recursion
        lineRel(dist, -dist);
    }
    timer() {
        dist = dist0;
        for (i = level; i > 0; i--) {
            dist /= 2;
        }
        goToXY(x_pos, y_pos);
        sierpA(level);  // start recursion
        lineRel(dist, dist);
        sierpB(level);  // start recursion
        lineRel(-dist, dist);
        sierpC(level);  // start recursion
        lineRel(-dist, -dist);
        sierpD(level);  // start recursion
        lineRel(dist, -dist);
    }
}
